

	DEFINE_OPCODE(op_new_object) {
        /* new_object dst(r)

           Constructs a new empty Object instance using the original
           constructor, and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = JSValue(constructEmptyObject(executeData.m_callFrame));

        executeData.m_vPC += OPCODE_LENGTH(op_new_object);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_new_array) {
        /* new_array dst(r) firstArg(r) argCount(n)

           Constructs a new Array instance using the original
           constructor, and puts the result in register dst.
           The array will contain argCount elements with values
           taken from registers starting at register firstArg.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int firstArg = executeData.m_vPC[2].u.operand;
        int argCount = executeData.m_vPC[3].u.operand;
        ArgList args(executeData.m_callFrame->registers() + firstArg, argCount);
        executeData.m_callFrame->uncheckedR(dst) = JSValue(constructArray(executeData.m_callFrame, args));

        executeData.m_vPC += OPCODE_LENGTH(op_new_array);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_new_array_buffer) {
        /* new_array_buffer dst(r) index(n) argCount(n)
         
         Constructs a new Array instance using the original
         constructor, and puts the result in register dst.
         The array be initialized with the values from constantBuffer[index]
         */
        int dst = executeData.m_vPC[1].u.operand;
        int firstArg = executeData.m_vPC[2].u.operand;
        int argCount = executeData.m_vPC[3].u.operand;
        ArgList args(executeData.m_codeBlock->constantBuffer(firstArg), argCount);
        executeData.m_callFrame->uncheckedR(dst) = JSValue(constructArray(executeData.m_callFrame, args));
        
        executeData.m_vPC += OPCODE_LENGTH(op_new_array);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_new_regexp) {
        /* new_regexp dst(r) regExp(re)

           Constructs a new RegExp instance using the original
           constructor from regexp regExp, and puts the result in
           register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        RegExp* regExp = executeData.m_codeBlock->regexp(executeData.m_vPC[2].u.operand);
        if (!regExp->isValid()) {
            executeData.m_exceptionValue = createSyntaxError(executeData.m_callFrame, "Invalid flags supplied to RegExp constructor.");
            VM_THROW();
        }
        executeData.m_callFrame->uncheckedR(dst) = JSValue(RegExpObject::create(*executeData.m_globalData, executeData.m_callFrame->lexicalGlobalObject(), executeData.m_callFrame->scopeChain()->globalObject->regExpStructure(), regExp));

        executeData.m_vPC += OPCODE_LENGTH(op_new_regexp);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_mov) {
        /* mov dst(r) src(r)

           Copies register src to register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        
        executeData.m_callFrame->uncheckedR(dst) = executeData.m_callFrame->r(src);

        executeData.m_vPC += OPCODE_LENGTH(op_mov);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_eq) {
        /* eq dst(r) src1(r) src2(r)

           Checks whether register src1 and register src2 are equal,
           as with the ECMAScript '==' operator, and puts the result
           as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(src1.asInt32() == src2.asInt32());
        else {
            JSValue result = jsBoolean(JSValue::equalSlowCase(executeData.m_callFrame, src1, src2));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_eq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_eq_null) {
        /* eq_null dst(r) src(r)

           Checks whether register src is null, as with the ECMAScript '!='
           operator, and puts the result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();

        if (src.isUndefinedOrNull()) {
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(true);
            executeData.m_vPC += OPCODE_LENGTH(op_eq_null);
            NEXT_INSTRUCTION();
        }
        
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(src.isCell() && src.asCell()->structure()->typeInfo().masqueradesAsUndefined());
        executeData.m_vPC += OPCODE_LENGTH(op_eq_null);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_neq) {
        /* neq dst(r) src1(r) src2(r)

           Checks whether register src1 and register src2 are not
           equal, as with the ECMAScript '!=' operator, and puts the
           result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(src1.asInt32() != src2.asInt32());
        else {
            JSValue result = jsBoolean(!JSValue::equalSlowCase(executeData.m_callFrame, src1, src2));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_neq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_neq_null) {
        /* neq_null dst(r) src(r)

           Checks whether register src is not null, as with the ECMAScript '!='
           operator, and puts the result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();

        if (src.isUndefinedOrNull()) {
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(false);
            executeData.m_vPC += OPCODE_LENGTH(op_neq_null);
            NEXT_INSTRUCTION();
        }
        
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(!src.isCell() || !src.asCell()->structure()->typeInfo().masqueradesAsUndefined());
        executeData.m_vPC += OPCODE_LENGTH(op_neq_null);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_stricteq) {
        /* stricteq dst(r) src1(r) src2(r)

           Checks whether register src1 and register src2 are strictly
           equal, as with the ECMAScript '===' operator, and puts the
           result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        bool result = JSValue::strictEqual(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(result);

        executeData.m_vPC += OPCODE_LENGTH(op_stricteq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_nstricteq) {
        /* nstricteq dst(r) src1(r) src2(r)

           Checks whether register src1 and register src2 are not
           strictly equal, as with the ECMAScript '!==' operator, and
           puts the result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        bool result = !JSValue::strictEqual(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(result);

        executeData.m_vPC += OPCODE_LENGTH(op_nstricteq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_less) {
        /* less dst(r) src1(r) src2(r)

           Checks whether register src1 is less than register src2, as
           with the ECMAScript '<' operator, and puts the result as
           a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        JSValue result = jsBoolean(jsLess<true>(executeData.m_callFrame, src1, src2));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_less);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_lesseq) {
        /* lesseq dst(r) src1(r) src2(r)

           Checks whether register src1 is less than or equal to
           register src2, as with the ECMAScript '<=' operator, and
           puts the result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        JSValue result = jsBoolean(jsLessEq<true>(executeData.m_callFrame, src1, src2));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_lesseq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_greater) {
        /* greater dst(r) src1(r) src2(r)

           Checks whether register src1 is greater than register src2, as
           with the ECMAScript '>' operator, and puts the result as
           a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        JSValue result = jsBoolean(jsLess<false>(executeData.m_callFrame, src2, src1));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_greater);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_greatereq) {
        /* greatereq dst(r) src1(r) src2(r)

           Checks whether register src1 is greater than or equal to
           register src2, as with the ECMAScript '>=' operator, and
           puts the result as a boolean in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        JSValue result = jsBoolean(jsLessEq<false>(executeData.m_callFrame, src2, src1));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_greatereq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_pre_inc) {
        /* pre_inc srcDst(r)

           Converts register srcDst to number, adds one, and puts the result
           back in register srcDst.
        */
        int srcDst = executeData.m_vPC[1].u.operand;
        JSValue v = executeData.m_callFrame->r(srcDst).jsValue();
        if (v.isInt32() && v.asInt32() < INT_MAX)
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(v.asInt32() + 1);
        else {
            JSValue result = jsNumber(v.toNumber(executeData.m_callFrame) + 1);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(srcDst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_pre_inc);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_pre_dec) {
        /* pre_dec srcDst(r)

           Converts register srcDst to number, subtracts one, and puts the result
           back in register srcDst.
        */
        int srcDst = executeData.m_vPC[1].u.operand;
        JSValue v = executeData.m_callFrame->r(srcDst).jsValue();
        if (v.isInt32() && v.asInt32() > INT_MIN)
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(v.asInt32() - 1);
        else {
            JSValue result = jsNumber(v.toNumber(executeData.m_callFrame) - 1);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(srcDst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_pre_dec);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_post_inc) {
        /* post_inc dst(r) srcDst(r)

           Converts register srcDst to number. The number itself is
           written to register dst, and the number plus one is written
           back to register srcDst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int srcDst = executeData.m_vPC[2].u.operand;
        JSValue v = executeData.m_callFrame->r(srcDst).jsValue();
        if (v.isInt32() && v.asInt32() < INT_MAX) {
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(v.asInt32() + 1);
            executeData.m_callFrame->uncheckedR(dst) = v;
        } else {
            JSValue number = executeData.m_callFrame->r(srcDst).jsValue().toJSNumber(executeData.m_callFrame);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(number.uncheckedGetNumber() + 1);
            executeData.m_callFrame->uncheckedR(dst) = number;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_post_inc);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_post_dec) {
        /* post_dec dst(r) srcDst(r)

           Converts register srcDst to number. The number itself is
           written to register dst, and the number minus one is written
           back to register srcDst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int srcDst = executeData.m_vPC[2].u.operand;
        JSValue v = executeData.m_callFrame->r(srcDst).jsValue();
        if (v.isInt32() && v.asInt32() > INT_MIN) {
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(v.asInt32() - 1);
            executeData.m_callFrame->uncheckedR(dst) = v;
        } else {
            JSValue number = executeData.m_callFrame->r(srcDst).jsValue().toJSNumber(executeData.m_callFrame);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(srcDst) = jsNumber(number.uncheckedGetNumber() - 1);
            executeData.m_callFrame->uncheckedR(dst) = number;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_post_dec);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_to_jsnumber) {
        /* to_jsnumber dst(r) src(r)

           Converts register src to number, and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;

        JSValue srcVal = executeData.m_callFrame->r(src).jsValue();

        if (LIKELY(srcVal.isNumber()))
            executeData.m_callFrame->uncheckedR(dst) = executeData.m_callFrame->r(src);
        else {
            JSValue result = srcVal.toJSNumber(executeData.m_callFrame);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_to_jsnumber);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_negate) {
        /* negate dst(r) src(r)

           Converts register src to number, negates it, and puts the
           result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        if (src.isInt32() && (src.asInt32() & 0x7fffffff)) // non-zero and no overflow
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(-src.asInt32());
        else {
            JSValue result = jsNumber(-src.toNumber(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_negate);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_add) {
        /* add dst(r) src1(r) src2(r)

           Adds register src1 and register src2, and puts the result
           in register dst. (JS add may be string concatenation or
           numeric add, depending on the types of the operands.)
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32() && !(src1.asInt32() | (src2.asInt32() & 0xc0000000))) // no overflow
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() + src2.asInt32());
        else {
            JSValue result = jsAdd(executeData.m_callFrame, src1, src2);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }
        executeData.m_vPC += OPCODE_LENGTH(op_add);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_mul) {
        /* mul dst(r) src1(r) src2(r)

           Multiplies register src1 and register src2 (converted to
           numbers), and puts the product in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32() && !(src1.asInt32() | src2.asInt32() >> 15)) // no overflow
                executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() * src2.asInt32());
        else {
            JSValue result = jsNumber(src1.toNumber(executeData.m_callFrame) * src2.toNumber(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_mul);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_div) {
        /* div dst(r) dividend(r) divisor(r)

           Divides register dividend (converted to number) by the
           register divisor (converted to number), and puts the
           quotient in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue dividend = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue divisor = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();

        JSValue result = jsNumber(dividend.toNumber(executeData.m_callFrame) / divisor.toNumber(executeData.m_callFrame));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_div);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_mod) {
        /* mod dst(r) dividend(r) divisor(r)

           Divides register dividend (converted to number) by
           register divisor (converted to number), and puts the
           remainder in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue dividend = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue divisor = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();

        if (dividend.isInt32() && divisor.isInt32() && divisor.asInt32() != 0) {
            JSValue result = jsNumber(dividend.asInt32() % divisor.asInt32());
            ASSERT(result);
            executeData.m_callFrame->uncheckedR(dst) = result;
            executeData.m_vPC += OPCODE_LENGTH(op_mod);
            NEXT_INSTRUCTION();
        }

        // Conversion to double must happen outside the call to fmod since the
        // order of argument evaluation is not guaranteed.
        double d1 = dividend.toNumber(executeData.m_callFrame);
        double d2 = divisor.toNumber(executeData.m_callFrame);
        JSValue result = jsNumber(fmod(d1, d2));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;
        executeData.m_vPC += OPCODE_LENGTH(op_mod);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_sub) {
        /* sub dst(r) src1(r) src2(r)

           Subtracts register src2 (converted to number) from register
           src1 (converted to number), and puts the difference in
           register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32() && !(src1.asInt32() | (src2.asInt32() & 0xc0000000))) // no overflow
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() - src2.asInt32());
        else {
            JSValue result = jsNumber(src1.toNumber(executeData.m_callFrame) - src2.toNumber(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }
        executeData.m_vPC += OPCODE_LENGTH(op_sub);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_lshift) {
        /* lshift dst(r) val(r) shift(r)

           Performs left shift of register val (converted to int32) by
           register shift (converted to uint32), and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue val = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue shift = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();

        if (val.isInt32() && shift.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(val.asInt32() << (shift.asInt32() & 0x1f));
        else {
            JSValue result = jsNumber((val.toInt32(executeData.m_callFrame)) << (shift.toUInt32(executeData.m_callFrame) & 0x1f));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_lshift);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_rshift) {
        /* rshift dst(r) val(r) shift(r)

           Performs arithmetic right shift of register val (converted
           to int32) by register shift (converted to
           uint32), and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue val = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue shift = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();

        if (val.isInt32() && shift.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(val.asInt32() >> (shift.asInt32() & 0x1f));
        else {
            JSValue result = jsNumber((val.toInt32(executeData.m_callFrame)) >> (shift.toUInt32(executeData.m_callFrame) & 0x1f));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_rshift);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_urshift) {
        /* rshift dst(r) val(r) shift(r)

           Performs logical right shift of register val (converted
           to uint32) by register shift (converted to
           uint32), and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue val = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue shift = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (val.isUInt32() && shift.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(val.asInt32() >> (shift.asInt32() & 0x1f));
        else {
            JSValue result = jsNumber((val.toUInt32(executeData.m_callFrame)) >> (shift.toUInt32(executeData.m_callFrame) & 0x1f));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_urshift);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_bitand) {
        /* bitand dst(r) src1(r) src2(r)

           Computes bitwise AND of register src1 (converted to int32)
           and register src2 (converted to int32), and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() & src2.asInt32());
        else {
            JSValue result = jsNumber(src1.toInt32(executeData.m_callFrame) & src2.toInt32(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_bitand);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_bitxor) {
        /* bitxor dst(r) src1(r) src2(r)

           Computes bitwise XOR of register src1 (converted to int32)
           and register src2 (converted to int32), and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() ^ src2.asInt32());
        else {
            JSValue result = jsNumber(src1.toInt32(executeData.m_callFrame) ^ src2.toInt32(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_bitxor);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_bitor) {
        /* bitor dst(r) src1(r) src2(r)

           Computes bitwise OR of register src1 (converted to int32)
           and register src2 (converted to int32), and puts the
           result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (src1.isInt32() && src2.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(src1.asInt32() | src2.asInt32());
        else {
            JSValue result = jsNumber(src1.toInt32(executeData.m_callFrame) | src2.toInt32(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }

        executeData.m_vPC += OPCODE_LENGTH(op_bitor);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_bitnot) {
        /* bitnot dst(r) src(r)

           Computes bitwise NOT of register src1 (converted to int32),
           and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        JSValue src = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        if (src.isInt32())
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(~src.asInt32());
        else {
            JSValue result = jsNumber(~src.toInt32(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        }
        executeData.m_vPC += OPCODE_LENGTH(op_bitnot);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_not) {
        /* not dst(r) src(r)

           Computes logical NOT of register src (converted to
           boolean), and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        JSValue result = jsBoolean(!executeData.m_callFrame->r(src).jsValue().toBoolean(executeData.m_callFrame));
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;

        executeData.m_vPC += OPCODE_LENGTH(op_not);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_check_has_instance) {
        /* check_has_instance constructor(r)

           Check 'constructor' is an object with the internal property
           [HasInstance] (i.e. is a function ... *shakes head sadly at
           JSC API*). Raises an exception if register constructor is not
           an valid parameter for instanceof.
        */
        int base = executeData.m_vPC[1].u.operand;
        JSValue baseVal = executeData.m_callFrame->r(base).jsValue();

        if (isInvalidParamForInstanceOf(executeData.m_callFrame, baseVal, executeData.m_exceptionValue))
            VM_THROW();

        executeData.m_vPC += OPCODE_LENGTH(op_check_has_instance);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_instanceof) {
        /* instanceof dst(r) value(r) constructor(r) constructorProto(r)

           Tests whether register value is an instance of register
           constructor, and puts the boolean result in register
           dst. Register constructorProto must contain the "prototype"
           property (not the actual prototype) of the object in
           register constructor. This lookup is separated so that
           polymorphic inline caching can apply.

           Raises an exception if register constructor is not an
           object.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int value = executeData.m_vPC[2].u.operand;
        int base = executeData.m_vPC[3].u.operand;
        int baseProto = executeData.m_vPC[4].u.operand;

        JSValue baseVal = executeData.m_callFrame->r(base).jsValue();

        ASSERT(!isInvalidParamForInstanceOf(executeData.m_callFrame, baseVal, executeData.m_exceptionValue));

        bool result = asObject(baseVal)->hasInstance(executeData.m_callFrame, executeData.m_callFrame->r(value).jsValue(), executeData.m_callFrame->r(baseProto).jsValue());
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(result);

        executeData.m_vPC += OPCODE_LENGTH(op_instanceof);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_typeof) {
        /* typeof dst(r) src(r)

           Determines the type string for src according to ECMAScript
           rules, and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = JSValue(jsTypeStringForValue(executeData.m_callFrame, executeData.m_callFrame->r(src).jsValue()));

        executeData.m_vPC += OPCODE_LENGTH(op_typeof);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_undefined) {
        /* is_undefined dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "undefined", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        JSValue v = executeData.m_callFrame->r(src).jsValue();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(v.isCell() ? v.asCell()->structure()->typeInfo().masqueradesAsUndefined() : v.isUndefined());

        executeData.m_vPC += OPCODE_LENGTH(op_is_undefined);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_boolean) {
        /* is_boolean dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "boolean", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(executeData.m_callFrame->r(src).jsValue().isBoolean());

        executeData.m_vPC += OPCODE_LENGTH(op_is_boolean);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_number) {
        /* is_number dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "number", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(executeData.m_callFrame->r(src).jsValue().isNumber());

        executeData.m_vPC += OPCODE_LENGTH(op_is_number);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_string) {
        /* is_string dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "string", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(executeData.m_callFrame->r(src).jsValue().isString());

        executeData.m_vPC += OPCODE_LENGTH(op_is_string);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_object) {
        /* is_object dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "object", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(jsIsObjectType(executeData.m_callFrame->r(src).jsValue()));

        executeData.m_vPC += OPCODE_LENGTH(op_is_object);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_is_function) {
        /* is_function dst(r) src(r)

           Determines whether the type string for src according to
           the ECMAScript rules is "function", and puts the result
           in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(jsIsFunctionType(executeData.m_callFrame->r(src).jsValue()));

        executeData.m_vPC += OPCODE_LENGTH(op_is_function);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_in) {
        /* in dst(r) property(r) base(r)

           Tests whether register base has a property named register
           property, and puts the boolean result in register dst.

           Raises an exception if register constructor is not an
           object.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int base = executeData.m_vPC[3].u.operand;

        JSValue baseVal = executeData.m_callFrame->r(base).jsValue();
        if (isInvalidParamForIn(executeData.m_callFrame, baseVal, executeData.m_exceptionValue))
            VM_THROW();

        JSObject* baseObj = asObject(baseVal);

        JSValue propName = executeData.m_callFrame->r(property).jsValue();

        uint32_t i;
        if (propName.getUInt32(i))
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(baseObj->hasProperty(executeData.m_callFrame, i));
        else {
            Identifier property(executeData.m_callFrame, propName.toString(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = jsBoolean(baseObj->hasProperty(executeData.m_callFrame, property));
        }

        executeData.m_vPC += OPCODE_LENGTH(op_in);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve) {
        /* resolve dst(r) property(id)

           Looks up the property named by identifier property in the
           scope chain, and writes the resulting value to register
           dst. If the property is not found, raises an exception.
        */
        if (UNLIKELY(!resolve(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();

        executeData.m_vPC += OPCODE_LENGTH(op_resolve);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_skip) {
        /* resolve_skip dst(r) property(id) skip(n)

         Looks up the property named by identifier property in the
         scope chain skipping the top 'skip' levels, and writes the resulting
         value to register dst. If the property is not found, raises an exception.
         */
        if (UNLIKELY(!resolveSkip(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();

        executeData.m_vPC += OPCODE_LENGTH(op_resolve_skip);

        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_global) {
        /* resolve_skip dst(r) globalObject(c) property(id) structure(sID) offset(n)
         
           Performs a dynamic property lookup for the given property, on the provided
           global object.  If structure matches the Structure of the global then perform
           a fast lookup using the case offset, otherwise fall back to a full resolve and
           cache the new structure and offset
         */
        if (UNLIKELY(!resolveGlobal(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();
        
        executeData.m_vPC += OPCODE_LENGTH(op_resolve_global);
        
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_global_dynamic) {
        /* resolve_skip dst(r) globalObject(c) property(id) structure(sID) offset(n), depth(n)
         
         Performs a dynamic property lookup for the given property, on the provided
         global object.  If structure matches the Structure of the global then perform
         a fast lookup using the case offset, otherwise fall back to a full resolve and
         cache the new structure and offset.
         
         This walks through n levels of the scope chain to verify that none of those levels
         in the scope chain include dynamically added properties.
         */
        if (UNLIKELY(!resolveGlobalDynamic(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();
        
        executeData.m_vPC += OPCODE_LENGTH(op_resolve_global_dynamic);
        
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_global_var) {
        /* get_global_var dst(r) globalObject(c) index(n)

           Gets the global var at global slot index and places it in register dst.
         */
        int dst = executeData.m_vPC[1].u.operand;
        JSGlobalObject* scope = executeData.m_codeBlock->globalObject();
        ASSERT(scope->isGlobalObject());
        int index = executeData.m_vPC[2].u.operand;

        executeData.m_callFrame->uncheckedR(dst) = scope->registerAt(index).get();
        executeData.m_vPC += OPCODE_LENGTH(op_get_global_var);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_global_var) {
        /* put_global_var globalObject(c) index(n) value(r)
         
           Puts value into global slot index.
         */
        JSGlobalObject* scope = executeData.m_codeBlock->globalObject();
        ASSERT(scope->isGlobalObject());
        int index = executeData.m_vPC[1].u.operand;
        int value = executeData.m_vPC[2].u.operand;
        
        scope->registerAt(index).set(*executeData.m_globalData, scope, executeData.m_callFrame->r(value).jsValue());
        executeData.m_vPC += OPCODE_LENGTH(op_put_global_var);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_scoped_var) {
        /* get_scoped_var dst(r) index(n) skip(n)

         Loads the contents of the index-th local from the scope skip nodes from
         the top of the scope chain, and places it in register dst.
         */
        int dst = executeData.m_vPC[1].u.operand;
        int index = executeData.m_vPC[2].u.operand;
        int skip = executeData.m_vPC[3].u.operand;

        ScopeChainNode* scopeChain = executeData.m_callFrame->scopeChain();
        ScopeChainIterator iter = scopeChain->begin();
        ScopeChainIterator end = scopeChain->end();
        ASSERT(iter != end);
        ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());
        bool checkTopLevel = executeData.m_codeBlock->codeType() == FunctionCode && executeData.m_codeBlock->needsFullScopeChain();
        ASSERT(skip || !checkTopLevel);
    if (checkTopLevel && skip--) {
            if (executeData.m_callFrame->r(executeData.m_codeBlock->activationRegister()).jsValue())
                ++iter;
        }
        while (skip--) {
            ++iter;
            ASSERT(iter != end);
        }
        ASSERT((*iter)->isVariableObject());
        JSVariableObject* scope = static_cast<JSVariableObject*>(iter->get());
        executeData.m_callFrame->uncheckedR(dst) = scope->registerAt(index).get();
        ASSERT(executeData.m_callFrame->r(dst).jsValue());
        executeData.m_vPC += OPCODE_LENGTH(op_get_scoped_var);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_scoped_var) {
        /* put_scoped_var index(n) skip(n) value(r)

         */
        int index = executeData.m_vPC[1].u.operand;
        int skip = executeData.m_vPC[2].u.operand;
        int value = executeData.m_vPC[3].u.operand;

        ScopeChainNode* scopeChain = executeData.m_callFrame->scopeChain();
        ScopeChainIterator iter = scopeChain->begin();
        ScopeChainIterator end = scopeChain->end();
        ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());
        ASSERT(iter != end);
        bool checkTopLevel = executeData.m_codeBlock->codeType() == FunctionCode && executeData.m_codeBlock->needsFullScopeChain();
        ASSERT(skip || !checkTopLevel);
    if (checkTopLevel && skip--) {
            if (executeData.m_callFrame->r(executeData.m_codeBlock->activationRegister()).jsValue())
                ++iter;
        }
        while (skip--) {
            ++iter;
            ASSERT(iter != end);
        }

        ASSERT((*iter)->isVariableObject());
        JSVariableObject* scope = static_cast<JSVariableObject*>(iter->get());
        ASSERT(executeData.m_callFrame->r(value).jsValue());
        scope->registerAt(index).set(*executeData.m_globalData, scope, executeData.m_callFrame->r(value).jsValue());
        executeData.m_vPC += OPCODE_LENGTH(op_put_scoped_var);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_base) {
        /* resolve_base dst(r) property(id) isStrict(bool)

           Searches the scope chain for an object containing
           identifier property, and if one is found, writes it to
           register dst. If none is found and isStrict is false, the
           outermost scope (which will be the global object) is
           stored in register dst.
        */
        resolveBase(executeData.m_callFrame, executeData.m_vPC);
        CHECK_FOR_EXCEPTION();

        executeData.m_vPC += OPCODE_LENGTH(op_resolve_base);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_ensure_property_exists) {
        /* ensure_property_exists base(r) property(id)

           Throws an exception if property does not exist on base
         */
        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        
        JSValue baseVal = executeData.m_callFrame->r(base).jsValue();
        JSObject* baseObject = asObject(baseVal);
        PropertySlot slot(baseVal);
        if (!baseObject->getPropertySlot(executeData.m_callFrame, ident, slot)) {
            executeData.m_exceptionValue = createErrorForInvalidGlobalAssignment(executeData.m_callFrame, ident.ustring());
            VM_THROW();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_ensure_property_exists);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_with_base) {
        /* resolve_with_base baseDst(r) propDst(r) property(id)

           Searches the scope chain for an object containing
           identifier property, and if one is found, writes it to
           register srcDst, and the retrieved property value to register
           propDst. If the property is not found, raises an exception.

           This is more efficient than doing resolve_base followed by
           resolve, or resolve_base followed by get_by_id, as it
           avoids duplicate hash lookups.
        */
        if (UNLIKELY(!resolveBaseAndProperty(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();

        executeData.m_vPC += OPCODE_LENGTH(op_resolve_with_base);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_resolve_with_this) {
        /* resolve_with_this thisDst(r) propDst(r) property(id)

           Searches the scope chain for an object containing
           identifier property, and if one is found, writes the
           retrieved property value to register propDst, and the
           this object to pass in a call to thisDst.

           If the property is not found, raises an exception.
        */
        if (UNLIKELY(!resolveThisAndProperty(executeData.m_callFrame, executeData.m_vPC, executeData.m_exceptionValue)))
            VM_THROW();

        executeData.m_vPC += OPCODE_LENGTH(op_resolve_with_this);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_by_id) {
        /* get_by_id dst(r) base(r) property(id) structure(sID) nop(n) nop(n) nop(n)

           Generic property access: Gets the property named by identifier
           property from the value base, and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;

        Identifier& ident = executeData.m_codeBlock->identifier(property);
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        PropertySlot slot(baseValue);
        JSValue result = baseValue.get(executeData.m_callFrame, ident, slot);
        CHECK_FOR_EXCEPTION();

        tryCacheGetByID(executeData.m_callFrame, executeData.m_codeBlock, executeData.m_vPC, baseValue, ident, slot);

        executeData.m_callFrame->uncheckedR(dst) = result;
        executeData.m_vPC += OPCODE_LENGTH(op_get_by_id);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_by_id_self) {
        /* op_get_by_id_self dst(r) base(r) property(id) structure(sID) offset(n) nop(n) nop(n)

           Cached property access: Attempts to get a cached property from the
           value base. If the cache misses, op_get_by_id_self reverts to
           op_get_by_id.
        */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();

        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();

            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(baseCell->isObject());
                JSObject* baseObject = asObject(baseCell);
                int dst = executeData.m_vPC[1].u.operand;
                int offset = executeData.m_vPC[5].u.operand;

                ASSERT(baseObject->get(executeData.m_callFrame, executeData.m_codeBlock->identifier(executeData.m_vPC[3].u.operand)) == baseObject->getDirectOffset(offset));
                executeData.m_callFrame->uncheckedR(dst) = JSValue(baseObject->getDirectOffset(offset));

                executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_self);
                NEXT_INSTRUCTION();
            }
        }

        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_by_id_proto) {
        /* op_get_by_id_proto dst(r) base(r) property(id) structure(sID) prototypeStructure(sID) offset(n) nop(n)

           Cached property access: Attempts to get a cached property from the
           value base's prototype. If the cache misses, op_get_by_id_proto
           reverts to op_get_by_id.
        */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();

        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();

            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(structure->prototypeForLookup(executeData.m_callFrame).isObject());
                JSObject* protoObject = asObject(structure->prototypeForLookup(executeData.m_callFrame));
                Structure* prototypeStructure = executeData.m_vPC[5].u.structure.get();

                if (LIKELY(protoObject->structure() == prototypeStructure)) {
                    int dst = executeData.m_vPC[1].u.operand;
                    int offset = executeData.m_vPC[6].u.operand;

                    ASSERT(protoObject->get(executeData.m_callFrame, executeData.m_codeBlock->identifier(executeData.m_vPC[3].u.operand)) == protoObject->getDirectOffset(offset));
                    ASSERT(baseValue.get(executeData.m_callFrame, executeData.m_codeBlock->identifier(executeData.m_vPC[3].u.operand)) == protoObject->getDirectOffset(offset));
                    executeData.m_callFrame->uncheckedR(dst) = JSValue(protoObject->getDirectOffset(offset));

                    executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_proto);
                    NEXT_INSTRUCTION();
                }
            }
        }

        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_id_getter_proto);
#endif
    DEFINE_OPCODE(op_get_by_id_getter_proto) {
        /* op_get_by_id_getter_proto dst(r) base(r) property(id) structure(sID) prototypeStructure(sID) offset(n) nop(n)
         
         Cached property access: Attempts to get a cached getter property from the
         value base's prototype. If the cache misses, op_get_by_id_getter_proto
         reverts to op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(structure->prototypeForLookup(executeData.m_callFrame).isObject());
                JSObject* protoObject = asObject(structure->prototypeForLookup(executeData.m_callFrame));
                Structure* prototypeStructure = executeData.m_vPC[5].u.structure.get();
                
                if (LIKELY(protoObject->structure() == prototypeStructure)) {
                    int dst = executeData.m_vPC[1].u.operand;
                    int offset = executeData.m_vPC[6].u.operand;
                    if (GetterSetter* getterSetter = asGetterSetter(protoObject->getDirectOffset(offset).asCell())) {
                        JSObject* getter = getterSetter->getter();
                        CallData callData;
                        CallType callType = getter->getCallData(callData);
                        JSValue result = call(executeData.m_callFrame, getter, callType, callData, asObject(baseCell), ArgList());
                        CHECK_FOR_EXCEPTION();
                        executeData.m_callFrame->uncheckedR(dst) = result;
                    } else
                        executeData.m_callFrame->uncheckedR(dst) = jsUndefined();
                    executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_getter_proto);
                    NEXT_INSTRUCTION();
                }
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_id_getter_proto:
#endif
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_id_custom_proto);
#endif
    DEFINE_OPCODE(op_get_by_id_custom_proto) {
        /* op_get_by_id_custom_proto dst(r) base(r) property(id) structure(sID) prototypeStructure(sID) offset(n) nop(n)
         
         Cached property access: Attempts to use a cached named property getter
         from the value base's prototype. If the cache misses, op_get_by_id_custom_proto
         reverts to op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(structure->prototypeForLookup(executeData.m_callFrame).isObject());
                JSObject* protoObject = asObject(structure->prototypeForLookup(executeData.m_callFrame));
                Structure* prototypeStructure = executeData.m_vPC[5].u.structure.get();
                
                if (LIKELY(protoObject->structure() == prototypeStructure)) {
                    int dst = executeData.m_vPC[1].u.operand;
                    int property = executeData.m_vPC[3].u.operand;
                    Identifier& ident = executeData.m_codeBlock->identifier(property);
                    
                    PropertySlot::GetValueFunc getter = executeData.m_vPC[6].u.getterFunc;
                    JSValue result = getter(executeData.m_callFrame, protoObject, ident);
                    CHECK_FOR_EXCEPTION();
                    executeData.m_callFrame->uncheckedR(dst) = result;
                    executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_custom_proto);
                    NEXT_INSTRUCTION();
                }
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
	}
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_id_custom_proto:
#endif
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_get_by_id_chain);
#endif
    DEFINE_OPCODE(op_get_by_id_chain) {
        /* op_get_by_id_chain dst(r) base(r) property(id) structure(sID) structureChain(chain) count(n) offset(n)

           Cached property access: Attempts to get a cached property from the
           value base's prototype chain. If the cache misses, op_get_by_id_chain
           reverts to op_get_by_id.
        */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();

        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();

            if (LIKELY(baseCell->structure() == structure)) {
                WriteBarrier<Structure>* it = executeData.m_vPC[5].u.structureChain->head();
                size_t count = executeData.m_vPC[6].u.operand;
                WriteBarrier<Structure>* end = it + count;

                while (true) {
                    JSObject* baseObject = asObject(baseCell->structure()->prototypeForLookup(executeData.m_callFrame));

                    if (UNLIKELY(baseObject->structure() != (*it).get()))
                        break;

                    if (++it == end) {
                        int dst = executeData.m_vPC[1].u.operand;
                        int offset = executeData.m_vPC[7].u.operand;

                        ASSERT(baseObject->get(executeData.m_callFrame, executeData.m_codeBlock->identifier(executeData.m_vPC[3].u.operand)) == baseObject->getDirectOffset(offset));
                        ASSERT(baseValue.get(executeData.m_callFrame, executeData.m_codeBlock->identifier(executeData.m_vPC[3].u.operand)) == baseObject->getDirectOffset(offset));
                        executeData.m_callFrame->uncheckedR(dst) = JSValue(baseObject->getDirectOffset(offset));

                        executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_chain);
                        NEXT_INSTRUCTION();
                    }

                    // Update baseCell, so that next time around the loop we'll pick up the prototype's prototype.
                    baseCell = baseObject;
                }
            }
        }

        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_get_by_id_chain:
    goto *(&&skip_id_getter_self);
#endif
    DEFINE_OPCODE(op_get_by_id_getter_self) {
        /* op_get_by_id_self dst(r) base(r) property(id) structure(sID) offset(n) nop(n) nop(n)
         
         Cached property access: Attempts to get a cached property from the
         value base. If the cache misses, op_get_by_id_getter_self reverts to
         op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(baseCell->isObject());
                JSObject* baseObject = asObject(baseCell);
                int dst = executeData.m_vPC[1].u.operand;
                int offset = executeData.m_vPC[5].u.operand;

                if (GetterSetter* getterSetter = asGetterSetter(baseObject->getDirectOffset(offset).asCell())) {
                    JSObject* getter = getterSetter->getter();
                    CallData callData;
                    CallType callType = getter->getCallData(callData);
                    JSValue result = call(executeData.m_callFrame, getter, callType, callData, baseObject, ArgList());
                    CHECK_FOR_EXCEPTION();
                    executeData.m_callFrame->uncheckedR(dst) = result;
                } else
                    executeData.m_callFrame->uncheckedR(dst) = jsUndefined();

                executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_getter_self);
                NEXT_INSTRUCTION();
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_id_getter_self:
#endif
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_id_custom_self);
#endif
    DEFINE_OPCODE(op_get_by_id_custom_self) {
        /* op_get_by_id_custom_self dst(r) base(r) property(id) structure(sID) offset(n) nop(n) nop(n)
         
         Cached property access: Attempts to use a cached named property getter
         from the value base. If the cache misses, op_get_by_id_custom_self reverts to
         op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(baseCell->isObject());
                int dst = executeData.m_vPC[1].u.operand;
                int property = executeData.m_vPC[3].u.operand;
                Identifier& ident = executeData.m_codeBlock->identifier(property);

                PropertySlot::GetValueFunc getter = executeData.m_vPC[5].u.getterFunc;
                JSValue result = getter(executeData.m_callFrame, baseValue, ident);
                CHECK_FOR_EXCEPTION();
                executeData.m_callFrame->uncheckedR(dst) = result;
                executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_custom_self);
                NEXT_INSTRUCTION();
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
skip_id_custom_self:
#endif
    DEFINE_OPCODE(op_get_by_id_generic) {
        /* op_get_by_id_generic dst(r) base(r) property(id) nop(sID) nop(n) nop(n) nop(n)

           Generic property access: Gets the property named by identifier
           property from the value base, and puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;

        Identifier& ident = executeData.m_codeBlock->identifier(property);
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        PropertySlot slot(baseValue);
        JSValue result = baseValue.get(executeData.m_callFrame, ident, slot);
        CHECK_FOR_EXCEPTION();

        executeData.m_callFrame->uncheckedR(dst) = result;
        executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_generic);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_id_getter_chain);
#endif
    DEFINE_OPCODE(op_get_by_id_getter_chain) {
        /* op_get_by_id_getter_chain dst(r) base(r) property(id) structure(sID) structureChain(chain) count(n) offset(n)
         
         Cached property access: Attempts to get a cached property from the
         value base's prototype chain. If the cache misses, op_get_by_id_getter_chain
         reverts to op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                WriteBarrier<Structure>* it = executeData.m_vPC[5].u.structureChain->head();
                size_t count = executeData.m_vPC[6].u.operand;
                WriteBarrier<Structure>* end = it + count;
                
                while (true) {
                    JSObject* baseObject = asObject(baseCell->structure()->prototypeForLookup(executeData.m_callFrame));
                    
                    if (UNLIKELY(baseObject->structure() != (*it).get()))
                        break;
                    
                    if (++it == end) {
                        int dst = executeData.m_vPC[1].u.operand;
                        int offset = executeData.m_vPC[7].u.operand;
                        if (GetterSetter* getterSetter = asGetterSetter(baseObject->getDirectOffset(offset).asCell())) {
                            JSObject* getter = getterSetter->getter();
                            CallData callData;
                            CallType callType = getter->getCallData(callData);
                            JSValue result = call(executeData.m_callFrame, getter, callType, callData, baseValue, ArgList());
                            CHECK_FOR_EXCEPTION();
                            executeData.m_callFrame->uncheckedR(dst) = result;
                        } else
                            executeData.m_callFrame->uncheckedR(dst) = jsUndefined();
                        executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_getter_chain);
                        NEXT_INSTRUCTION();
                    }
                    
                    // Update baseCell, so that next time around the loop we'll pick up the prototype's prototype.
                    baseCell = baseObject;
                }
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_id_getter_chain:
#endif
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    goto *(&&skip_id_custom_chain);
#endif
    DEFINE_OPCODE(op_get_by_id_custom_chain) {
        /* op_get_by_id_custom_chain dst(r) base(r) property(id) structure(sID) structureChain(chain) count(n) offset(n)
         
         Cached property access: Attempts to use a cached named property getter on the
         value base's prototype chain. If the cache misses, op_get_by_id_custom_chain
         reverts to op_get_by_id.
         */
        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();
            
            if (LIKELY(baseCell->structure() == structure)) {
                WriteBarrier<Structure>* it = executeData.m_vPC[5].u.structureChain->head();
                size_t count = executeData.m_vPC[6].u.operand;
                WriteBarrier<Structure>* end = it + count;
                
                while (true) {
                    JSObject* baseObject = asObject(baseCell->structure()->prototypeForLookup(executeData.m_callFrame));
                    
                    if (UNLIKELY(baseObject->structure() != (*it).get()))
                        break;
                    
                    if (++it == end) {
                        int dst = executeData.m_vPC[1].u.operand;
                        int property = executeData.m_vPC[3].u.operand;
                        Identifier& ident = executeData.m_codeBlock->identifier(property);
                        
                        PropertySlot::GetValueFunc getter = executeData.m_vPC[7].u.getterFunc;
                        JSValue result = getter(executeData.m_callFrame, baseObject, ident);
                        CHECK_FOR_EXCEPTION();
                        executeData.m_callFrame->uncheckedR(dst) = result;
                        executeData.m_vPC += OPCODE_LENGTH(op_get_by_id_custom_chain);
                        NEXT_INSTRUCTION();
                    }
                    
                    // Update baseCell, so that next time around the loop we'll pick up the prototype's prototype.
                    baseCell = baseObject;
                }
            }
        }
        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_id_custom_chain:
    goto *(&&skip_get_array_length);
#endif
    DEFINE_OPCODE(op_get_array_length) {
        /* op_get_array_length dst(r) base(r) property(id) nop(sID) nop(n) nop(n) nop(n)

           Cached property access: Gets the length of the array in register base,
           and puts the result in register dst. If register base does not hold
           an array, op_get_array_length reverts to op_get_by_id.
        */

        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        if (LIKELY(isJSArray(executeData.m_globalData, baseValue))) {
            int dst = executeData.m_vPC[1].u.operand;
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(asArray(baseValue)->length());
            executeData.m_vPC += OPCODE_LENGTH(op_get_array_length);
            NEXT_INSTRUCTION();
        }

        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_get_array_length:
    goto *(&&skip_get_string_length);
#endif
    DEFINE_OPCODE(op_get_string_length) {
        /* op_get_string_length dst(r) base(r) property(id) nop(sID) nop(n) nop(n) nop(n)

           Cached property access: Gets the length of the string in register base,
           and puts the result in register dst. If register base does not hold
           a string, op_get_string_length reverts to op_get_by_id.
        */

        int base = executeData.m_vPC[2].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        if (LIKELY(isJSString(executeData.m_globalData, baseValue))) {
            int dst = executeData.m_vPC[1].u.operand;
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(asString(baseValue)->length());
            executeData.m_vPC += OPCODE_LENGTH(op_get_string_length);
            NEXT_INSTRUCTION();
        }

        uncacheGetByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
    skip_get_string_length:
    goto *(&&skip_put_by_id);
#endif
    DEFINE_OPCODE(op_put_by_id) {
        /* put_by_id base(r) property(id) value(r) nop(n) nop(n) nop(n) nop(n) direct(b)

           Generic property access: Sets the property named by identifier
           property, belonging to register base, to register value.

           Unlike many opcodes, this one does not write any output to
           the register file.

           The "direct" flag should only be set this put_by_id is to initialize
           an object literal.
        */

        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int value = executeData.m_vPC[3].u.operand;
        int direct = executeData.m_vPC[8].u.operand;

        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        PutPropertySlot slot(executeData.m_codeBlock->isStrictMode());
        if (direct) {
            baseValue.putDirect(executeData.m_callFrame, ident, executeData.m_callFrame->r(value).jsValue(), slot);
            ASSERT(slot.base() == baseValue);
        } else
            baseValue.put(executeData.m_callFrame, ident, executeData.m_callFrame->r(value).jsValue(), slot);
        CHECK_FOR_EXCEPTION();

        tryCachePutByID(executeData.m_callFrame, executeData.m_codeBlock, executeData.m_vPC, baseValue, slot);

        executeData.m_vPC += OPCODE_LENGTH(op_put_by_id);
        NEXT_INSTRUCTION();
    }
#if USE(GCC_COMPUTED_GOTO_WORKAROUND)
      skip_put_by_id:
#endif
    DEFINE_OPCODE(op_put_by_id_transition) {
        /* op_put_by_id_transition base(r) property(id) value(r) oldStructure(sID) newStructure(sID) structureChain(chain) offset(n) direct(b)
         
           Cached property access: Attempts to set a new property with a cached transition
           property named by identifier property, belonging to register base,
           to register value. If the cache misses, op_put_by_id_transition
           reverts to op_put_by_id_generic.
         
           Unlike many opcodes, this one does not write any output to
           the register file.
         */
        int base = executeData.m_vPC[1].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        
        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* oldStructure = executeData.m_vPC[4].u.structure.get();
            Structure* newStructure = executeData.m_vPC[5].u.structure.get();
            
            if (LIKELY(baseCell->structure() == oldStructure)) {
                ASSERT(baseCell->isObject());
                JSObject* baseObject = asObject(baseCell);
                int direct = executeData.m_vPC[8].u.operand;
                
                if (!direct) {
                    WriteBarrier<Structure>* it = executeData.m_vPC[6].u.structureChain->head();

                    JSValue proto = baseObject->structure()->prototypeForLookup(executeData.m_callFrame);
                    while (!proto.isNull()) {
                        if (UNLIKELY(asObject(proto)->structure() != (*it).get())) {
                            uncachePutByID(executeData.m_codeBlock, executeData.m_vPC);
                            NEXT_INSTRUCTION();
                        }
                        ++it;
                        proto = asObject(proto)->structure()->prototypeForLookup(executeData.m_callFrame);
                    }
                }
                baseObject->transitionTo(*executeData.m_globalData, newStructure);

                int value = executeData.m_vPC[3].u.operand;
                unsigned offset = executeData.m_vPC[7].u.operand;
                ASSERT(baseObject->offsetForLocation(baseObject->getDirectLocation(*executeData.m_globalData, executeData.m_codeBlock->identifier(executeData.m_vPC[2].u.operand))) == offset);
                baseObject->putDirectOffset(executeData.m_callFrame->globalData(), offset, executeData.m_callFrame->r(value).jsValue());

                executeData.m_vPC += OPCODE_LENGTH(op_put_by_id_transition);
                NEXT_INSTRUCTION();
            }
        }
        
        uncachePutByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_by_id_replace) {
        /* op_put_by_id_replace base(r) property(id) value(r) structure(sID) offset(n) nop(n) nop(n) direct(b)

           Cached property access: Attempts to set a pre-existing, cached
           property named by identifier property, belonging to register base,
           to register value. If the cache misses, op_put_by_id_replace
           reverts to op_put_by_id.

           Unlike many opcodes, this one does not write any output to
           the register file.
        */
        int base = executeData.m_vPC[1].u.operand;
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();

        if (LIKELY(baseValue.isCell())) {
            JSCell* baseCell = baseValue.asCell();
            Structure* structure = executeData.m_vPC[4].u.structure.get();

            if (LIKELY(baseCell->structure() == structure)) {
                ASSERT(baseCell->isObject());
                JSObject* baseObject = asObject(baseCell);
                int value = executeData.m_vPC[3].u.operand;
                unsigned offset = executeData.m_vPC[5].u.operand;
                
                ASSERT(baseObject->offsetForLocation(baseObject->getDirectLocation(*executeData.m_globalData, executeData.m_codeBlock->identifier(executeData.m_vPC[2].u.operand))) == offset);
                baseObject->putDirectOffset(executeData.m_callFrame->globalData(), offset, executeData.m_callFrame->r(value).jsValue());

                executeData.m_vPC += OPCODE_LENGTH(op_put_by_id_replace);
                NEXT_INSTRUCTION();
            }
        }

        uncachePutByID(executeData.m_codeBlock, executeData.m_vPC);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_by_id_generic) {
        /* op_put_by_id_generic base(r) property(id) value(r) nop(n) nop(n) nop(n) nop(n) direct(b)

           Generic property access: Sets the property named by identifier
           property, belonging to register base, to register value.

           Unlike many opcodes, this one does not write any output to
           the register file.
        */
        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int value = executeData.m_vPC[3].u.operand;
        int direct = executeData.m_vPC[8].u.operand;

        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        PutPropertySlot slot(executeData.m_codeBlock->isStrictMode());
        if (direct) {
            baseValue.putDirect(executeData.m_callFrame, ident, executeData.m_callFrame->r(value).jsValue(), slot);
            ASSERT(slot.base() == baseValue);
        } else
            baseValue.put(executeData.m_callFrame, ident, executeData.m_callFrame->r(value).jsValue(), slot);
        CHECK_FOR_EXCEPTION();

        executeData.m_vPC += OPCODE_LENGTH(op_put_by_id_generic);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_del_by_id) {
        /* del_by_id dst(r) base(r) property(id)

           Converts register base to Object, deletes the property
           named by identifier property from the object, and writes a
           boolean indicating success (if true) or failure (if false)
           to register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;

        JSObject* baseObj = executeData.m_callFrame->r(base).jsValue().toObject(executeData.m_callFrame);
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        bool result = baseObj->deleteProperty(executeData.m_callFrame, ident);
        if (!result && executeData.m_codeBlock->isStrictMode()) {
            executeData.m_exceptionValue = createTypeError(executeData.m_callFrame, "Unable to delete property.");
            VM_THROW();
        }
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(result);
        executeData.m_vPC += OPCODE_LENGTH(op_del_by_id);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_by_pname) {
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;
        int expected = executeData.m_vPC[4].u.operand;
        int iter = executeData.m_vPC[5].u.operand;
        int i = executeData.m_vPC[6].u.operand;

        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        JSPropertyNameIterator* it = executeData.m_callFrame->r(iter).propertyNameIterator();
        JSValue subscript = executeData.m_callFrame->r(property).jsValue();
        JSValue expectedSubscript = executeData.m_callFrame->r(expected).jsValue();
        int index = executeData.m_callFrame->r(i).i() - 1;
        JSValue result;
        int offset = 0;
        if (subscript == expectedSubscript && baseValue.isCell() && (baseValue.asCell()->structure() == it->cachedStructure()) && it->getOffset(index, offset)) {
            executeData.m_callFrame->uncheckedR(dst) = JSValue(asObject(baseValue)->getDirectOffset(offset));
            executeData.m_vPC += OPCODE_LENGTH(op_get_by_pname);
            NEXT_INSTRUCTION();
        }
        {
            Identifier propertyName(executeData.m_callFrame, subscript.toString(executeData.m_callFrame));
            result = baseValue.get(executeData.m_callFrame, propertyName);
        }
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;
        executeData.m_vPC += OPCODE_LENGTH(op_get_by_pname);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_arguments_length) {
        int dst = executeData.m_vPC[1].u.operand;
        int argumentsRegister = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;
        JSValue arguments = executeData.m_callFrame->r(argumentsRegister).jsValue();
        if (arguments) {
            Identifier& ident = executeData.m_codeBlock->identifier(property);
            PropertySlot slot(arguments);
            JSValue result = arguments.get(executeData.m_callFrame, ident, slot);
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(dst) = result;
        } else
            executeData.m_callFrame->uncheckedR(dst) = jsNumber(executeData.m_callFrame->argumentCount());

        executeData.m_vPC += OPCODE_LENGTH(op_get_arguments_length);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_argument_by_val) {
        int dst = executeData.m_vPC[1].u.operand;
        int argumentsRegister = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;
        JSValue arguments = executeData.m_callFrame->r(argumentsRegister).jsValue();
        JSValue subscript = executeData.m_callFrame->r(property).jsValue();
        if (!arguments && subscript.isUInt32() && subscript.asUInt32() < executeData.m_callFrame->argumentCount()) {
            unsigned arg = subscript.asUInt32() + 1;
            unsigned numParameters = executeData.m_callFrame->codeBlock()->m_numParameters;
            if (arg < numParameters)
                executeData.m_callFrame->uncheckedR(dst) = executeData.m_callFrame->r(arg - RegisterFile::CallFrameHeaderSize - numParameters);
            else
                executeData.m_callFrame->uncheckedR(dst) = executeData.m_callFrame->r(arg - RegisterFile::CallFrameHeaderSize - numParameters - executeData.m_callFrame->argumentCount() - 1);
            executeData.m_vPC += OPCODE_LENGTH(op_get_argument_by_val);
            NEXT_INSTRUCTION();
        }
        if (!arguments) {
            Arguments* arguments = Arguments::create(*executeData.m_globalData, executeData.m_callFrame);
            executeData.m_callFrame->uncheckedR(argumentsRegister) = JSValue(arguments);
            executeData.m_callFrame->uncheckedR(unmodifiedArgumentsRegister(argumentsRegister)) = JSValue(arguments);
        }
        // fallthrough
#ifdef ENABLE_FUNC_INTERPRETER
		// Because these are now individual functions and not case statements, we have to explicitly call the next function
		return Interpreter::func_op_get_by_val(executeData);
#endif
    }
    DEFINE_OPCODE(op_get_by_val) {
        /* get_by_val dst(r) base(r) property(r)

           Converts register base to Object, gets the property named
           by register property from the object, and puts the result
           in register dst. property is nominally converted to string
           but numbers are treated more efficiently.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;
        
        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        JSValue subscript = executeData.m_callFrame->r(property).jsValue();

        JSValue result;

        if (LIKELY(subscript.isUInt32())) {
            uint32_t i = subscript.asUInt32();
            if (isJSArray(executeData.m_globalData, baseValue)) {
                JSArray* jsArray = asArray(baseValue);
                if (jsArray->canGetIndex(i))
                    result = jsArray->getIndex(i);
                else
                    result = jsArray->JSArray::get(executeData.m_callFrame, i);
            } else if (isJSString(executeData.m_globalData, baseValue) && asString(baseValue)->canGetIndex(i))
                result = asString(baseValue)->getIndex(executeData.m_callFrame, i);
            else if (isJSByteArray(executeData.m_globalData, baseValue) && asByteArray(baseValue)->canAccessIndex(i))
                result = asByteArray(baseValue)->getIndex(executeData.m_callFrame, i);
            else
                result = baseValue.get(executeData.m_callFrame, i);
        } else {
            Identifier property(executeData.m_callFrame, subscript.toString(executeData.m_callFrame));
            result = baseValue.get(executeData.m_callFrame, property);
        }

        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = result;
        executeData.m_vPC += OPCODE_LENGTH(op_get_by_val);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_by_val) {
        /* put_by_val base(r) property(r) value(r)

           Sets register value on register base as the property named
           by register property. Base is converted to object
           first. register property is nominally converted to string
           but numbers are treated more efficiently.

           Unlike many opcodes, this one does not write any output to
           the register file.
        */
        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int value = executeData.m_vPC[3].u.operand;

        JSValue baseValue = executeData.m_callFrame->r(base).jsValue();
        JSValue subscript = executeData.m_callFrame->r(property).jsValue();

        if (LIKELY(subscript.isUInt32())) {
            uint32_t i = subscript.asUInt32();
            if (isJSArray(executeData.m_globalData, baseValue)) {
                JSArray* jsArray = asArray(baseValue);
                if (jsArray->canSetIndex(i))
                    jsArray->setIndex(*executeData.m_globalData, i, executeData.m_callFrame->r(value).jsValue());
                else
                    jsArray->JSArray::put(executeData.m_callFrame, i, executeData.m_callFrame->r(value).jsValue());
            } else if (isJSByteArray(executeData.m_globalData, baseValue) && asByteArray(baseValue)->canAccessIndex(i)) {
                JSByteArray* jsByteArray = asByteArray(baseValue);
                double dValue = 0;
                JSValue jsValue = executeData.m_callFrame->r(value).jsValue();
                if (jsValue.isInt32())
                    jsByteArray->setIndex(i, jsValue.asInt32());
                else if (jsValue.getNumber(dValue))
                    jsByteArray->setIndex(i, dValue);
                else
                    baseValue.put(executeData.m_callFrame, i, jsValue);
            } else
                baseValue.put(executeData.m_callFrame, i, executeData.m_callFrame->r(value).jsValue());
        } else {
            Identifier property(executeData.m_callFrame, subscript.toString(executeData.m_callFrame));
            if (!executeData.m_globalData->exception) { // Don't put to an object if toString threw an exception.
                PutPropertySlot slot(executeData.m_codeBlock->isStrictMode());
                baseValue.put(executeData.m_callFrame, property, executeData.m_callFrame->r(value).jsValue(), slot);
            }
        }

        CHECK_FOR_EXCEPTION();
        executeData.m_vPC += OPCODE_LENGTH(op_put_by_val);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_del_by_val) {
        /* del_by_val dst(r) base(r) property(r)

           Converts register base to Object, deletes the property
           named by register property from the object, and writes a
           boolean indicating success (if true) or failure (if false)
           to register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int property = executeData.m_vPC[3].u.operand;

        JSObject* baseObj = executeData.m_callFrame->r(base).jsValue().toObject(executeData.m_callFrame); // may throw

        JSValue subscript = executeData.m_callFrame->r(property).jsValue();
        bool result;
        uint32_t i;
        if (subscript.getUInt32(i))
            result = baseObj->deleteProperty(executeData.m_callFrame, i);
        else {
            CHECK_FOR_EXCEPTION();
            Identifier property(executeData.m_callFrame, subscript.toString(executeData.m_callFrame));
            CHECK_FOR_EXCEPTION();
            result = baseObj->deleteProperty(executeData.m_callFrame, property);
        }
        if (!result && executeData.m_codeBlock->isStrictMode()) {
            executeData.m_exceptionValue = createTypeError(executeData.m_callFrame, "Unable to delete property.");
            VM_THROW();
        }
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(dst) = jsBoolean(result);
        executeData.m_vPC += OPCODE_LENGTH(op_del_by_val);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_by_index) {
        /* put_by_index base(r) property(n) value(r)

           Sets register value on register base as the property named
           by the immediate number property. Base is converted to
           object first.

           Unlike many opcodes, this one does not write any output to
           the register file.

           This opcode is mainly used to initialize array literals.
        */
        int base = executeData.m_vPC[1].u.operand;
        unsigned property = executeData.m_vPC[2].u.operand;
        int value = executeData.m_vPC[3].u.operand;

        executeData.m_callFrame->r(base).jsValue().put(executeData.m_callFrame, property, executeData.m_callFrame->r(value).jsValue());

        executeData.m_vPC += OPCODE_LENGTH(op_put_by_index);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop) {
        /* loop target(offset)
         
           Jumps unconditionally to offset target from the current
           instruction.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
         */
#if ENABLE(OPCODE_STATS)
        OpcodeStats::resetLastInstruction();
#endif
        int target = executeData.m_vPC[1].u.operand;
        CHECK_FOR_TIMEOUT();
        executeData.m_vPC += target;
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jmp) {
        /* jmp target(offset)

           Jumps unconditionally to offset target from the current
           instruction.
        */
#if ENABLE(OPCODE_STATS)
        OpcodeStats::resetLastInstruction();
#endif
        int target = executeData.m_vPC[1].u.operand;

        executeData.m_vPC += target;
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_true) {
        /* loop_if_true cond(r) target(offset)
         
           Jumps to offset target from the current instruction, if and
           only if register cond converts to boolean as true.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
         */
        int cond = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        if (executeData.m_callFrame->r(cond).jsValue().toBoolean(executeData.m_callFrame)) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_true);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_false) {
        /* loop_if_true cond(r) target(offset)
         
           Jumps to offset target from the current instruction, if and
           only if register cond converts to boolean as false.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
         */
        int cond = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        if (!executeData.m_callFrame->r(cond).jsValue().toBoolean(executeData.m_callFrame)) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_true);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jtrue) {
        /* jtrue cond(r) target(offset)

           Jumps to offset target from the current instruction, if and
           only if register cond converts to boolean as true.
        */
        int cond = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        if (executeData.m_callFrame->r(cond).jsValue().toBoolean(executeData.m_callFrame)) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jtrue);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jfalse) {
        /* jfalse cond(r) target(offset)

           Jumps to offset target from the current instruction, if and
           only if register cond converts to boolean as false.
        */
        int cond = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        if (!executeData.m_callFrame->r(cond).jsValue().toBoolean(executeData.m_callFrame)) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jfalse);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jeq_null) {
        /* jeq_null src(r) target(offset)

           Jumps to offset target from the current instruction, if and
           only if register src is null.
        */
        int src = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        JSValue srcValue = executeData.m_callFrame->r(src).jsValue();

        if (srcValue.isUndefinedOrNull() || (srcValue.isCell() && srcValue.asCell()->structure()->typeInfo().masqueradesAsUndefined())) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jeq_null);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jneq_null) {
        /* jneq_null src(r) target(offset)

           Jumps to offset target from the current instruction, if and
           only if register src is not null.
        */
        int src = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        JSValue srcValue = executeData.m_callFrame->r(src).jsValue();

        if (!srcValue.isUndefinedOrNull() && (!srcValue.isCell() || !srcValue.asCell()->structure()->typeInfo().masqueradesAsUndefined())) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jneq_null);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jneq_ptr) {
        /* jneq_ptr src(r) ptr(jsCell) target(offset)
         
           Jumps to offset target from the current instruction, if the value r is equal
           to ptr, using pointer equality.
         */
        int src = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[3].u.operand;
        JSValue srcValue = executeData.m_callFrame->r(src).jsValue();
        if (srcValue != executeData.m_vPC[2].u.jsCell.get()) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jneq_ptr);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_less) {
        /* loop_if_less src1(r) src2(r) target(offset)

           Checks whether register src1 is less than register src2, as
           with the ECMAScript '<' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is true.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
         */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLess<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_less);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_lesseq) {
        /* loop_if_lesseq src1(r) src2(r) target(offset)

           Checks whether register src1 is less than or equal to register
           src2, as with the ECMAScript '<=' operator, and then jumps to
           offset target from the current instruction, if and only if the 
           result of the comparison is true.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLessEq<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_lesseq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_greater) {
        /* loop_if_greater src1(r) src2(r) target(offset)

           Checks whether register src1 is greater than register src2, as
           with the ECMAScript '>' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is true.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
         */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLess<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_greater);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_loop_if_greatereq) {
        /* loop_if_greatereq src1(r) src2(r) target(offset)

           Checks whether register src1 is greater than or equal to register
           src2, as with the ECMAScript '>=' operator, and then jumps to
           offset target from the current instruction, if and only if the 
           result of the comparison is true.

           Additionally this loop instruction may terminate JS execution is
           the JS timeout is reached.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLessEq<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            CHECK_FOR_TIMEOUT();
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_loop_if_greatereq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jless) {
        /* jless src1(r) src2(r) target(offset)

           Checks whether register src1 is less than register src2, as
           with the ECMAScript '<' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is true.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLess<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jless);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jlesseq) {
        /* jlesseq src1(r) src2(r) target(offset)
         
         Checks whether register src1 is less than or equal to
         register src2, as with the ECMAScript '<=' operator,
         and then jumps to offset target from the current instruction,
         if and only if the result of the comparison is true.
         */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLessEq<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_jlesseq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jgreater) {
        /* jgreater src1(r) src2(r) target(offset)

           Checks whether register src1 is greater than register src2, as
           with the ECMAScript '>' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is true.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLess<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jgreater);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jgreatereq) {
        /* jgreatereq src1(r) src2(r) target(offset)
         
         Checks whether register src1 is greater than or equal to
         register src2, as with the ECMAScript '>=' operator,
         and then jumps to offset target from the current instruction,
         if and only if the result of the comparison is true.
         */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;
        
        bool result = jsLessEq<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }
        
        executeData.m_vPC += OPCODE_LENGTH(op_jgreatereq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jnless) {
        /* jnless src1(r) src2(r) target(offset)

           Checks whether register src1 is less than register src2, as
           with the ECMAScript '<' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is false.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLess<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (!result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jnless);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jnlesseq) {
        /* jnlesseq src1(r) src2(r) target(offset)

           Checks whether register src1 is less than or equal to
           register src2, as with the ECMAScript '<=' operator,
           and then jumps to offset target from the current instruction,
           if and only if theresult of the comparison is false.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLessEq<true>(executeData.m_callFrame, src1, src2);
        CHECK_FOR_EXCEPTION();
        
        if (!result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jnlesseq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jngreater) {
        /* jngreater src1(r) src2(r) target(offset)

           Checks whether register src1 is greater than register src2, as
           with the ECMAScript '>' operator, and then jumps to offset
           target from the current instruction, if and only if the 
           result of the comparison is false.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLess<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (!result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jngreater);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jngreatereq) {
        /* jngreatereq src1(r) src2(r) target(offset)

           Checks whether register src1 is greater than or equal to
           register src2, as with the ECMAScript '>=' operator,
           and then jumps to offset target from the current instruction,
           if and only if theresult of the comparison is false.
        */
        JSValue src1 = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue();
        JSValue src2 = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();
        int target = executeData.m_vPC[3].u.operand;

        bool result = jsLessEq<false>(executeData.m_callFrame, src2, src1);
        CHECK_FOR_EXCEPTION();
        
        if (!result) {
            executeData.m_vPC += target;
            NEXT_INSTRUCTION();
        }

        executeData.m_vPC += OPCODE_LENGTH(op_jngreatereq);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_switch_imm) {
        /* switch_imm tableIndex(n) defaultOffset(offset) scrutinee(r)

           Performs a range checked switch on the scrutinee value, using
           the tableIndex-th immediate switch jump table.  If the scrutinee value
           is an immediate number in the range covered by the referenced jump
           table, and the value at jumpTable[scrutinee value] is non-zero, then
           that value is used as the jump offset, otherwise defaultOffset is used.
         */
        int tableIndex = executeData.m_vPC[1].u.operand;
        int defaultOffset = executeData.m_vPC[2].u.operand;
        JSValue scrutinee = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (scrutinee.isInt32())
            executeData.m_vPC += executeData.m_codeBlock->immediateSwitchJumpTable(tableIndex).offsetForValue(scrutinee.asInt32(), defaultOffset);
        else {
            double value;
            int32_t intValue;
            if (scrutinee.getNumber(value) && ((intValue = static_cast<int32_t>(value)) == value))
                executeData.m_vPC += executeData.m_codeBlock->immediateSwitchJumpTable(tableIndex).offsetForValue(intValue, defaultOffset);
            else
                executeData.m_vPC += defaultOffset;
        }
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_switch_char) {
        /* switch_char tableIndex(n) defaultOffset(offset) scrutinee(r)

           Performs a range checked switch on the scrutinee value, using
           the tableIndex-th character switch jump table.  If the scrutinee value
           is a single character string in the range covered by the referenced jump
           table, and the value at jumpTable[scrutinee value] is non-zero, then
           that value is used as the jump offset, otherwise defaultOffset is used.
         */
        int tableIndex = executeData.m_vPC[1].u.operand;
        int defaultOffset = executeData.m_vPC[2].u.operand;
        JSValue scrutinee = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (!scrutinee.isString())
            executeData.m_vPC += defaultOffset;
        else {
            StringImpl* value = asString(scrutinee)->value(executeData.m_callFrame).impl();
            if (value->length() != 1)
                executeData.m_vPC += defaultOffset;
            else
                executeData.m_vPC += executeData.m_codeBlock->characterSwitchJumpTable(tableIndex).offsetForValue((*value)[0], defaultOffset);
        }
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_switch_string) {
        /* switch_string tableIndex(n) defaultOffset(offset) scrutinee(r)

           Performs a sparse hashmap based switch on the value in the scrutinee
           register, using the tableIndex-th string switch jump table.  If the 
           scrutinee value is a string that exists as a key in the referenced 
           jump table, then the value associated with the string is used as the 
           jump offset, otherwise defaultOffset is used.
         */
        int tableIndex = executeData.m_vPC[1].u.operand;
        int defaultOffset = executeData.m_vPC[2].u.operand;
        JSValue scrutinee = executeData.m_callFrame->r(executeData.m_vPC[3].u.operand).jsValue();
        if (!scrutinee.isString())
            executeData.m_vPC += defaultOffset;
        else 
            executeData.m_vPC += executeData.m_codeBlock->stringSwitchJumpTable(tableIndex).offsetForValue(asString(scrutinee)->value(executeData.m_callFrame).impl(), defaultOffset);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_new_func) {
        /* new_func dst(r) func(f)

           Constructs a new Function instance from function func and
           the current scope chain using the original Function
           constructor, using the rules for function declarations, and
           puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int func = executeData.m_vPC[2].u.operand;
        int shouldCheck = executeData.m_vPC[3].u.operand;
        ASSERT(executeData.m_codeBlock->codeType() != FunctionCode || !executeData.m_codeBlock->needsFullScopeChain() || executeData.m_callFrame->r(executeData.m_codeBlock->activationRegister()).jsValue());
        if (!shouldCheck || !executeData.m_callFrame->r(dst).jsValue())
            executeData.m_callFrame->uncheckedR(dst) = JSValue(executeData.m_codeBlock->functionDecl(func)->make(executeData.m_callFrame, executeData.m_callFrame->scopeChain()));

        executeData.m_vPC += OPCODE_LENGTH(op_new_func);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_new_func_exp) {
        /* new_func_exp dst(r) func(f)

           Constructs a new Function instance from function func and
           the current scope chain using the original Function
           constructor, using the rules for function expressions, and
           puts the result in register dst.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int funcIndex = executeData.m_vPC[2].u.operand;
        
        ASSERT(executeData.m_codeBlock->codeType() != FunctionCode || !executeData.m_codeBlock->needsFullScopeChain() || executeData.m_callFrame->r(executeData.m_codeBlock->activationRegister()).jsValue());
        FunctionExecutable* function = executeData.m_codeBlock->functionExpr(funcIndex);
        JSFunction* func = function->make(executeData.m_callFrame, executeData.m_callFrame->scopeChain());

        /* 
            The Identifier in a FunctionExpression can be referenced from inside
            the FunctionExpression's FunctionBody to allow the function to call
            itself recursively. However, unlike in a FunctionDeclaration, the
            Identifier in a FunctionExpression cannot be referenced from and
            does not affect the scope enclosing the FunctionExpression.
         */
        if (!function->name().isNull()) {
            JSStaticScopeObject* functionScopeObject = JSStaticScopeObject::create(executeData.m_callFrame, function->name(), func, ReadOnly | DontDelete);
            func->setScope(*executeData.m_globalData, func->scope()->push(functionScopeObject));
        }

        executeData.m_callFrame->uncheckedR(dst) = JSValue(func);

        executeData.m_vPC += OPCODE_LENGTH(op_new_func_exp);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_call_eval) {
        /* call_eval func(r) argCount(n) registerOffset(n)

           Call a function named "eval" with no explicit "this" value
           (which may therefore be the eval operator). If register
           thisVal is the global object, and register func contains
           that global object's original global eval function, then
           perform the eval operator in local scope (interpreting
           the argument registers as for the "call"
           opcode). Otherwise, act exactly as the "call" opcode would.
         */

        int func = executeData.m_vPC[1].u.operand;
        int argCount = executeData.m_vPC[2].u.operand;
        int registerOffset = executeData.m_vPC[3].u.operand;
        
        ASSERT(executeData.m_codeBlock->codeType() != FunctionCode || !executeData.m_codeBlock->needsFullScopeChain() || executeData.m_callFrame->r(executeData.m_codeBlock->activationRegister()).jsValue());
        JSValue funcVal = executeData.m_callFrame->r(func).jsValue();

        if (isHostFunction(executeData.m_callFrame->globalData(), funcVal, globalFuncEval)) {
            Register* newCallFrame = executeData.m_callFrame->registers() + registerOffset;
            Register* argv = newCallFrame - RegisterFile::CallFrameHeaderSize - argCount;

            JSValue result = callEval(executeData.m_callFrame, executeData.m_registerFile, argv, argCount, registerOffset);
            if ((executeData.m_exceptionValue = executeData.m_globalData->exception))
                VM_THROW();
            executeData.m_functionReturnValue = result;

            executeData.m_vPC += OPCODE_LENGTH(op_call_eval);
            NEXT_INSTRUCTION();
        }

        // We didn't find the blessed version of eval, so process this
        // instruction as a normal function call.
        // fall through to op_call
#ifdef ENABLE_FUNC_INTERPRETER
		// Because these are now individual functions and not case statements, we have to explicitly call the next function
		return Interpreter::func_op_call(executeData);
#endif
    }
    DEFINE_OPCODE(op_call) {
        /* call func(r) argCount(n) registerOffset(n)

           Perform a function call.
           
           registerOffset is the distance the executeData.m_callFrame pointer should move
           before the VM initializes the new call frame's header.
           
           dst is where op_ret should store its result.
         */

        int func = executeData.m_vPC[1].u.operand;
        int argCount = executeData.m_vPC[2].u.operand;
        int registerOffset = executeData.m_vPC[3].u.operand;

        JSValue v = executeData.m_callFrame->r(func).jsValue();

        CallData callData;
        CallType callType = getCallData(v, callData);

        if (callType == CallTypeJS) {
            ScopeChainNode* callDataScopeChain = callData.js.scopeChain;

            JSObject* error = callData.js.functionExecutable->compileForCall(executeData.m_callFrame, callDataScopeChain);
            if (UNLIKELY(!!error)) {
                executeData.m_exceptionValue = error;
                VM_THROW();
            }

            CallFrame* previousCallFrame = executeData.m_callFrame;
            CodeBlock* newCodeBlock = &callData.js.functionExecutable->generatedBytecodeForCall();
            executeData.m_callFrame = slideRegisterWindowForCall(newCodeBlock, executeData.m_registerFile, executeData.m_callFrame, registerOffset, argCount);
            if (UNLIKELY(!executeData.m_callFrame)) {
                executeData.m_callFrame = previousCallFrame;
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }

            executeData.m_callFrame->init(newCodeBlock, executeData.m_vPC + OPCODE_LENGTH(op_call), callDataScopeChain, previousCallFrame, argCount, asFunction(v));
            executeData.m_codeBlock = newCodeBlock;
            ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());
            *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            executeData.m_vPC = newCodeBlock->instructions().begin();

#if ENABLE(OPCODE_STATS)
            OpcodeStats::resetLastInstruction();
#endif

            NEXT_INSTRUCTION();
        }

        if (callType == CallTypeHost) {
            ScopeChainNode* scopeChain = executeData.m_callFrame->scopeChain();
            CallFrame* newCallFrame = CallFrame::create(executeData.m_callFrame->registers() + registerOffset);
            if (!executeData.m_registerFile->grow(newCallFrame->registers())) {
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }

            newCallFrame->init(0, executeData.m_vPC + OPCODE_LENGTH(op_call), scopeChain, executeData.m_callFrame, argCount, asObject(v));
            JSValue returnValue;
            {
                *executeData.m_topCallFrameSlot = newCallFrame;
                SamplingTool::HostCallRecord callRecord(m_sampler.get());
                returnValue = JSValue::decode(callData.native.function(newCallFrame));
                *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            }
            CHECK_FOR_EXCEPTION();

            executeData.m_functionReturnValue = returnValue;

            executeData.m_vPC += OPCODE_LENGTH(op_call);
            NEXT_INSTRUCTION();
        }

        ASSERT(callType == CallTypeNone);

        executeData.m_exceptionValue = createNotAFunctionError(executeData.m_callFrame, v);
        VM_THROW();
    }
    DEFINE_OPCODE(op_load_varargs) {
        int argCountDst = executeData.m_vPC[1].u.operand;
        int argsOffset = executeData.m_vPC[2].u.operand;
        
        JSValue arguments = executeData.m_callFrame->r(argsOffset).jsValue();
        uint32_t argCount = 0;
        if (!arguments) {
            argCount = (uint32_t)(executeData.m_callFrame->argumentCount());
            if (argCount > Arguments::MaxArguments) {
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }
            int32_t sizeDelta = argsOffset + argCount + RegisterFile::CallFrameHeaderSize;
            Register* newEnd = executeData.m_callFrame->registers() + sizeDelta;
            if (!executeData.m_registerFile->grow(newEnd) || ((newEnd - executeData.m_callFrame->registers()) != sizeDelta)) {
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }
            ASSERT(!asFunction(executeData.m_callFrame->callee())->isHostFunction());
            int32_t expectedParams = asFunction(executeData.m_callFrame->callee())->jsExecutable()->parameterCount();
            int32_t inplaceArgs = min(static_cast<int32_t>(argCount), expectedParams);
            int32_t i = 0;
            Register* argStore = executeData.m_callFrame->registers() + argsOffset;

            // First step is to copy the "expected" parameters from their normal location relative to the callframe
            for (; i < inplaceArgs; i++)
                argStore[i] = executeData.m_callFrame->registers()[i - RegisterFile::CallFrameHeaderSize - expectedParams];
            // Then we copy any additional arguments that may be further up the stack ('-1' to account for 'this')
            for (; i < static_cast<int32_t>(argCount); i++)
                argStore[i] = executeData.m_callFrame->registers()[i - RegisterFile::CallFrameHeaderSize - expectedParams - static_cast<int32_t>(argCount) - 1];
        } else if (!arguments.isUndefinedOrNull()) {
            if (!arguments.isObject()) {
                executeData.m_exceptionValue = createInvalidParamError(executeData.m_callFrame, "Function.prototype.apply", arguments);
                VM_THROW();
            }
            if (asObject(arguments)->classInfo() == &Arguments::s_info) {
                Arguments* args = asArguments(arguments);
                argCount = args->numProvidedArguments(executeData.m_callFrame);
                if (argCount > Arguments::MaxArguments) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                int32_t sizeDelta = argsOffset + argCount + RegisterFile::CallFrameHeaderSize;
                Register* newEnd = executeData.m_callFrame->registers() + sizeDelta;
                if (!executeData.m_registerFile->grow(newEnd) || ((newEnd - executeData.m_callFrame->registers()) != sizeDelta)) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                args->copyToRegisters(executeData.m_callFrame, executeData.m_callFrame->registers() + argsOffset, argCount);
            } else if (isJSArray(&executeData.m_callFrame->globalData(), arguments)) {
                JSArray* array = asArray(arguments);
                argCount = array->length();
                if (argCount > Arguments::MaxArguments) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                int32_t sizeDelta = argsOffset + argCount + RegisterFile::CallFrameHeaderSize;
                Register* newEnd = executeData.m_callFrame->registers() + sizeDelta;
                if (!executeData.m_registerFile->grow(newEnd) || ((newEnd - executeData.m_callFrame->registers()) != sizeDelta)) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                array->copyToRegisters(executeData.m_callFrame, executeData.m_callFrame->registers() + argsOffset, argCount);
            } else {
                JSObject* argObject = asObject(arguments);
                argCount = argObject->get(executeData.m_callFrame, executeData.m_callFrame->propertyNames().length).toUInt32(executeData.m_callFrame);
                if (argCount > Arguments::MaxArguments) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                int32_t sizeDelta = argsOffset + argCount + RegisterFile::CallFrameHeaderSize;
                Register* newEnd = executeData.m_callFrame->registers() + sizeDelta;
                if (!executeData.m_registerFile->grow(newEnd) || ((newEnd - executeData.m_callFrame->registers()) != sizeDelta)) {
                    executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                    VM_THROW();
                }
                Register* argsBuffer = executeData.m_callFrame->registers() + argsOffset;
                for (uint32_t i = 0; i < argCount; ++i) {
                    argsBuffer[i] = asObject(arguments)->get(executeData.m_callFrame, i);
                    CHECK_FOR_EXCEPTION();
                }
            }
        }
        CHECK_FOR_EXCEPTION();
        executeData.m_callFrame->uncheckedR(argCountDst) = Register::withInt(argCount + 1);
        executeData.m_vPC += OPCODE_LENGTH(op_load_varargs);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_call_varargs) {
        /* call_varargs func(r) argCountReg(r) baseRegisterOffset(n)
         
         Perform a function call with a dynamic set of arguments.
         
         registerOffset is the distance the executeData.m_callFrame pointer should move
         before the VM initializes the new call frame's header, excluding
         space for arguments.
         
         dst is where op_ret should store its result.
         */
        
        int func = executeData.m_vPC[1].u.operand;
        int argCountReg = executeData.m_vPC[2].u.operand;
        int registerOffset = executeData.m_vPC[3].u.operand;
        
        JSValue v = executeData.m_callFrame->r(func).jsValue();
        int argCount = executeData.m_callFrame->r(argCountReg).i();
        registerOffset += argCount;
        CallData callData;
        CallType callType = getCallData(v, callData);
        
        if (callType == CallTypeJS) {
            ScopeChainNode* callDataScopeChain = callData.js.scopeChain;

            JSObject* error = callData.js.functionExecutable->compileForCall(executeData.m_callFrame, callDataScopeChain);
            if (UNLIKELY(!!error)) {
                executeData.m_exceptionValue = error;
                VM_THROW();
            }

            CallFrame* previousCallFrame = executeData.m_callFrame;
            CodeBlock* newCodeBlock = &callData.js.functionExecutable->generatedBytecodeForCall();
            executeData.m_callFrame = slideRegisterWindowForCall(newCodeBlock, executeData.m_registerFile, executeData.m_callFrame, registerOffset, argCount);
            if (UNLIKELY(!executeData.m_callFrame)) {
                executeData.m_callFrame = previousCallFrame;
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }

            executeData.m_callFrame->init(newCodeBlock, executeData.m_vPC + OPCODE_LENGTH(op_call_varargs), callDataScopeChain, previousCallFrame, argCount, asFunction(v));
            executeData.m_codeBlock = newCodeBlock;
            ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());
            *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            executeData.m_vPC = newCodeBlock->instructions().begin();
            
#if ENABLE(OPCODE_STATS)
            OpcodeStats::resetLastInstruction();
#endif
            
            NEXT_INSTRUCTION();
        }
        
        if (callType == CallTypeHost) {
            ScopeChainNode* scopeChain = executeData.m_callFrame->scopeChain();
            CallFrame* newCallFrame = CallFrame::create(executeData.m_callFrame->registers() + registerOffset);
            if (!executeData.m_registerFile->grow(newCallFrame->registers())) {
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }
            newCallFrame->init(0, executeData.m_vPC + OPCODE_LENGTH(op_call_varargs), scopeChain, executeData.m_callFrame, argCount, asObject(v));
            
            JSValue returnValue;
            {
                *executeData.m_topCallFrameSlot = newCallFrame;
                SamplingTool::HostCallRecord callRecord(m_sampler.get());
                returnValue = JSValue::decode(callData.native.function(newCallFrame));
                *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            }
            CHECK_FOR_EXCEPTION();
            
            executeData.m_functionReturnValue = returnValue;
            
            executeData.m_vPC += OPCODE_LENGTH(op_call_varargs);
            NEXT_INSTRUCTION();
        }
        
        ASSERT(callType == CallTypeNone);
        
        executeData.m_exceptionValue = createNotAFunctionError(executeData.m_callFrame, v);
        VM_THROW();
    }
    DEFINE_OPCODE(op_tear_off_activation) {
        /* tear_off_activation activation(r) arguments(r)

           Copy locals and named parameters from the register file to the heap.
           Point the bindings in 'activation' and 'arguments' to this new backing
           store. (Note that 'arguments' may not have been created. If created,
           'arguments' already holds a copy of any extra / unnamed parameters.)

           This opcode appears before op_ret in functions that require full scope chains.
        */

        int activation = executeData.m_vPC[1].u.operand;
        int arguments = executeData.m_vPC[2].u.operand;
        ASSERT(executeData.m_codeBlock->needsFullScopeChain());
        JSValue activationValue = executeData.m_callFrame->r(activation).jsValue();
        if (activationValue) {
            asActivation(activationValue)->copyRegisters(*executeData.m_globalData);

            if (JSValue argumentsValue = executeData.m_callFrame->r(unmodifiedArgumentsRegister(arguments)).jsValue()) {
                if (!executeData.m_codeBlock->isStrictMode())
                    asArguments(argumentsValue)->setActivation(*executeData.m_globalData, asActivation(activationValue));
            }
        } else if (JSValue argumentsValue = executeData.m_callFrame->r(unmodifiedArgumentsRegister(arguments)).jsValue()) {
            if (!executeData.m_codeBlock->isStrictMode())
                asArguments(argumentsValue)->copyRegisters(*executeData.m_globalData);
        }

        executeData.m_vPC += OPCODE_LENGTH(op_tear_off_activation);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_tear_off_arguments) {
        /* tear_off_arguments arguments(r)

           Copy named parameters from the register file to the heap. Point the
           bindings in 'arguments' to this new backing store. (Note that
           'arguments' may not have been created. If created, 'arguments' already
           holds a copy of any extra / unnamed parameters.)

           This opcode appears before op_ret in functions that don't require full
           scope chains, but do use 'arguments'.
        */

        int src1 = executeData.m_vPC[1].u.operand;
        ASSERT(!executeData.m_codeBlock->needsFullScopeChain() && executeData.m_codeBlock->ownerExecutable()->usesArguments());

        if (JSValue arguments = executeData.m_callFrame->r(unmodifiedArgumentsRegister(src1)).jsValue())
            asArguments(arguments)->copyRegisters(*executeData.m_globalData);

        executeData.m_vPC += OPCODE_LENGTH(op_tear_off_arguments);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_ret) {
        /* ret result(r)
           
           Return register result as the return value of the current
           function call, writing it into executeData.m_functionReturnValue.
           In addition, unwind one call frame and restore the scope
           chain, code block instruction pointer and register base
           to those of the calling function.
        */

        int result = executeData.m_vPC[1].u.operand;

        JSValue returnValue = executeData.m_callFrame->r(result).jsValue();

        executeData.m_vPC = executeData.m_callFrame->returnVPC();
        executeData.m_callFrame = executeData.m_callFrame->callerFrame();

        if (executeData.m_callFrame->hasHostCallFrameFlag())
            END_EXECUTE(returnValue);

        *executeData.m_topCallFrameSlot = executeData.m_callFrame;
        executeData.m_functionReturnValue = returnValue;
        executeData.m_codeBlock = executeData.m_callFrame->codeBlock();
        ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());

        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_call_put_result) {
        /* op_call_put_result result(r)
           
           Move call result from executeData.m_functionReturnValue to caller's
           expected return value register.
        */

        executeData.m_callFrame->uncheckedR(executeData.m_vPC[1].u.operand) = executeData.m_functionReturnValue;

        executeData.m_vPC += OPCODE_LENGTH(op_call_put_result);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_ret_object_or_this) {
        /* ret result(r)
           
           Return register result as the return value of the current
           function call, writing it into the caller's expected return
           value register. In addition, unwind one call frame and
           restore the scope chain, code block instruction pointer and
           register base to those of the calling function.
        */

        int result = executeData.m_vPC[1].u.operand;

        JSValue returnValue = executeData.m_callFrame->r(result).jsValue();

        if (UNLIKELY(!returnValue.isObject()))
            returnValue = executeData.m_callFrame->r(executeData.m_vPC[2].u.operand).jsValue();

        executeData.m_vPC = executeData.m_callFrame->returnVPC();
        executeData.m_callFrame = executeData.m_callFrame->callerFrame();

        if (executeData.m_callFrame->hasHostCallFrameFlag())
            END_EXECUTE(returnValue);

        *executeData.m_topCallFrameSlot = executeData.m_callFrame;
        executeData.m_functionReturnValue = returnValue;
        executeData.m_codeBlock = executeData.m_callFrame->codeBlock();
        ASSERT(executeData.m_codeBlock == executeData.m_callFrame->codeBlock());

        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_enter) {
        /* enter

           Initializes local variables to undefined. If the code block requires
           an activation, enter_with_activation is used instead.

           This opcode appears only at the beginning of a code block.
        */

        size_t i = 0;
        for (size_t count = executeData.m_codeBlock->m_numVars; i < count; ++i)
            executeData.m_callFrame->uncheckedR(i) = jsUndefined();

        executeData.m_vPC += OPCODE_LENGTH(op_enter);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_create_activation) {
        /* create_activation dst(r)

           If the activation object for this callframe has not yet been created,
           this creates it and writes it back to dst.
        */

        int activationReg = executeData.m_vPC[1].u.operand;
        if (!executeData.m_callFrame->r(activationReg).jsValue()) {
            JSActivation* activation = JSActivation::create(*executeData.m_globalData, executeData.m_callFrame, static_cast<FunctionExecutable*>(executeData.m_codeBlock->ownerExecutable()));
            executeData.m_callFrame->r(activationReg) = JSValue(activation);
            executeData.m_callFrame->setScopeChain(executeData.m_callFrame->scopeChain()->push(activation));
        }
        executeData.m_vPC += OPCODE_LENGTH(op_create_activation);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_callee) {
        /* op_get_callee callee(r)

           Move callee into a register.
        */

        executeData.m_callFrame->uncheckedR(executeData.m_vPC[1].u.operand) = JSValue(executeData.m_callFrame->callee());

        executeData.m_vPC += OPCODE_LENGTH(op_get_callee);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_create_this) {
        /* op_create_this this(r) proto(r)

           Allocate an object as 'this', fr use in construction.

           This opcode should only be used at the beginning of a code
           block.
        */

        int thisRegister = executeData.m_vPC[1].u.operand;
        int protoRegister = executeData.m_vPC[2].u.operand;

        JSFunction* constructor = asFunction(executeData.m_callFrame->callee());
#if !ASSERT_DISABLED
        ConstructData constructData;
        ASSERT(constructor->getConstructData(constructData) == ConstructTypeJS);
#endif

        Structure* structure;
        JSValue proto = executeData.m_callFrame->r(protoRegister).jsValue();
        if (proto.isObject())
            structure = asObject(proto)->inheritorID(executeData.m_callFrame->globalData());
        else
            structure = constructor->scope()->globalObject->emptyObjectStructure();
        executeData.m_callFrame->uncheckedR(thisRegister) = constructEmptyObject(executeData.m_callFrame, structure);

        executeData.m_vPC += OPCODE_LENGTH(op_create_this);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_convert_this) {
        /* convert_this this(r)

           Takes the value in the 'this' register, converts it to a
           value that is suitable for use as the 'this' value, and
           stores it in the 'this' register. This opcode is emitted
           to avoid doing the conversion in the caller unnecessarily.

           This opcode should only be used at the beginning of a code
           block.
        */

        int thisRegister = executeData.m_vPC[1].u.operand;
        JSValue thisVal = executeData.m_callFrame->r(thisRegister).jsValue();
        if (thisVal.isPrimitive())
            executeData.m_callFrame->uncheckedR(thisRegister) = JSValue(thisVal.toThisObject(executeData.m_callFrame));

        executeData.m_vPC += OPCODE_LENGTH(op_convert_this);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_init_lazy_reg) {
        /* init_lazy_reg dst(r)

           Initialises dst(r) to JSValue().

           This opcode appears only at the beginning of a code block.
         */
        int dst = executeData.m_vPC[1].u.operand;

        executeData.m_callFrame->uncheckedR(dst) = JSValue();
        executeData.m_vPC += OPCODE_LENGTH(op_init_lazy_reg);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_create_arguments) {
        /* create_arguments dst(r)

           Creates the 'arguments' object and places it in both the
           'arguments' call frame slot and the local 'arguments'
           register, if it has not already been initialised.
         */
        
        int dst = executeData.m_vPC[1].u.operand;

        if (!executeData.m_callFrame->r(dst).jsValue()) {
            Arguments* arguments = Arguments::create(*executeData.m_globalData, executeData.m_callFrame);
            executeData.m_callFrame->uncheckedR(dst) = JSValue(arguments);
            executeData.m_callFrame->uncheckedR(unmodifiedArgumentsRegister(dst)) = JSValue(arguments);
        }
        executeData.m_vPC += OPCODE_LENGTH(op_create_arguments);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_construct) {
        /* construct func(r) argCount(n) registerOffset(n) proto(r) thisRegister(r)

           Invoke register "func" as a constructor. For JS
           functions, the calling convention is exactly as for the
           "call" opcode, except that the "this" value is a newly
           created Object. For native constructors, no "this"
           value is passed. In either case, the argCount and registerOffset
           registers are interpreted as for the "call" opcode.

           Register proto must contain the prototype property of
           register func. This is to enable polymorphic inline
           caching of this lookup.
        */

        int func = executeData.m_vPC[1].u.operand;
        int argCount = executeData.m_vPC[2].u.operand;
        int registerOffset = executeData.m_vPC[3].u.operand;

        JSValue v = executeData.m_callFrame->r(func).jsValue();

        ConstructData constructData;
        ConstructType constructType = getConstructData(v, constructData);

        if (constructType == ConstructTypeJS) {
            ScopeChainNode* callDataScopeChain = constructData.js.scopeChain;

            JSObject* error = constructData.js.functionExecutable->compileForConstruct(executeData.m_callFrame, callDataScopeChain);
            if (UNLIKELY(!!error)) {
                executeData.m_exceptionValue = error;
                VM_THROW();
            }

            CallFrame* previousCallFrame = executeData.m_callFrame;
            CodeBlock* newCodeBlock = &constructData.js.functionExecutable->generatedBytecodeForConstruct();
            executeData.m_callFrame = slideRegisterWindowForCall(newCodeBlock, executeData.m_registerFile, executeData.m_callFrame, registerOffset, argCount);
            if (UNLIKELY(!executeData.m_callFrame)) {
                executeData.m_callFrame = previousCallFrame;
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }

            executeData.m_callFrame->init(newCodeBlock, executeData.m_vPC + OPCODE_LENGTH(op_construct), callDataScopeChain, previousCallFrame, argCount, asFunction(v));
            executeData.m_codeBlock = newCodeBlock;
            *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            executeData.m_vPC = newCodeBlock->instructions().begin();
#if ENABLE(OPCODE_STATS)
            OpcodeStats::resetLastInstruction();
#endif

            NEXT_INSTRUCTION();
        }

        if (constructType == ConstructTypeHost) {
            ScopeChainNode* scopeChain = executeData.m_callFrame->scopeChain();
            CallFrame* newCallFrame = CallFrame::create(executeData.m_callFrame->registers() + registerOffset);
            if (!executeData.m_registerFile->grow(newCallFrame->registers())) {
                executeData.m_exceptionValue = createStackOverflowError(executeData.m_callFrame);
                VM_THROW();
            }
            newCallFrame->init(0, executeData.m_vPC + OPCODE_LENGTH(op_construct), scopeChain, executeData.m_callFrame, argCount, asObject(v));

            JSValue returnValue;
            {
                *executeData.m_topCallFrameSlot = newCallFrame;
                SamplingTool::HostCallRecord callRecord(m_sampler.get());
                returnValue = JSValue::decode(constructData.native.function(newCallFrame));
                *executeData.m_topCallFrameSlot = executeData.m_callFrame;
            }
            CHECK_FOR_EXCEPTION();
            executeData.m_functionReturnValue = returnValue;

            executeData.m_vPC += OPCODE_LENGTH(op_construct);
            NEXT_INSTRUCTION();
        }

        ASSERT(constructType == ConstructTypeNone);

        executeData.m_exceptionValue = createNotAConstructorError(executeData.m_callFrame, v);
        VM_THROW();
    }
    DEFINE_OPCODE(op_strcat) {
        /* strcat dst(r) src(r) count(n)

           Construct a new String instance using the original
           constructor, and puts the result in register dst.
           The string will be the result of concatenating count
           strings with values taken from registers starting at
           register src.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;
        int count = executeData.m_vPC[3].u.operand;

        executeData.m_callFrame->uncheckedR(dst) = concatenateStrings(executeData.m_callFrame, &executeData.m_callFrame->registers()[src], count);
        CHECK_FOR_EXCEPTION();
        executeData.m_vPC += OPCODE_LENGTH(op_strcat);

        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_to_primitive) {
        int dst = executeData.m_vPC[1].u.operand;
        int src = executeData.m_vPC[2].u.operand;

        executeData.m_callFrame->uncheckedR(dst) = executeData.m_callFrame->r(src).jsValue().toPrimitive(executeData.m_callFrame);
        executeData.m_vPC += OPCODE_LENGTH(op_to_primitive);

        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_push_scope) {
        /* push_scope scope(r)

           Converts register scope to object, and pushes it onto the top
           of the current scope chain.  The contents of the register scope
           are replaced by the result of toObject conversion of the scope.
        */
        int scope = executeData.m_vPC[1].u.operand;
        JSValue v = executeData.m_callFrame->r(scope).jsValue();
        JSObject* o = v.toObject(executeData.m_callFrame);
        CHECK_FOR_EXCEPTION();

        executeData.m_callFrame->uncheckedR(scope) = JSValue(o);
        executeData.m_callFrame->setScopeChain(executeData.m_callFrame->scopeChain()->push(o));

        executeData.m_vPC += OPCODE_LENGTH(op_push_scope);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_pop_scope) {
        /* pop_scope

           Removes the top item from the current scope chain.
        */
        executeData.m_callFrame->setScopeChain(executeData.m_callFrame->scopeChain()->pop());

        executeData.m_vPC += OPCODE_LENGTH(op_pop_scope);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_get_pnames) {
        /* get_pnames dst(r) base(r) i(n) size(n) breakTarget(offset)

           Creates a property name list for register base and puts it
           in register dst, initializing i and size for iteration. If
           base is undefined or null, jumps to breakTarget.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int i = executeData.m_vPC[3].u.operand;
        int size = executeData.m_vPC[4].u.operand;
        int breakTarget = executeData.m_vPC[5].u.operand;

        JSValue v = executeData.m_callFrame->r(base).jsValue();
        if (v.isUndefinedOrNull()) {
            executeData.m_vPC += breakTarget;
            NEXT_INSTRUCTION();
        }

        JSObject* o = v.toObject(executeData.m_callFrame);
        Structure* structure = o->structure();
        JSPropertyNameIterator* jsPropertyNameIterator = structure->enumerationCache();
        if (!jsPropertyNameIterator || jsPropertyNameIterator->cachedPrototypeChain() != structure->prototypeChain(executeData.m_callFrame))
            jsPropertyNameIterator = JSPropertyNameIterator::create(executeData.m_callFrame, o);

        executeData.m_callFrame->uncheckedR(dst) = jsPropertyNameIterator;
        executeData.m_callFrame->uncheckedR(base) = JSValue(o);
        executeData.m_callFrame->uncheckedR(i) = Register::withInt(0);
        executeData.m_callFrame->uncheckedR(size) = Register::withInt(jsPropertyNameIterator->size());
        executeData.m_vPC += OPCODE_LENGTH(op_get_pnames);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_next_pname) {
        /* next_pname dst(r) base(r) i(n) size(n) iter(r) target(offset)

           Copies the next name from the property name list in
           register iter to dst, then jumps to offset target. If there are no
           names left, invalidates the iterator and continues to the next
           instruction.
        */
        int dst = executeData.m_vPC[1].u.operand;
        int base = executeData.m_vPC[2].u.operand;
        int i = executeData.m_vPC[3].u.operand;
        int size = executeData.m_vPC[4].u.operand;
        int iter = executeData.m_vPC[5].u.operand;
        int target = executeData.m_vPC[6].u.operand;

        JSPropertyNameIterator* it = executeData.m_callFrame->r(iter).propertyNameIterator();
        while (executeData.m_callFrame->r(i).i() != executeData.m_callFrame->r(size).i()) {
            JSValue key = it->get(executeData.m_callFrame, asObject(executeData.m_callFrame->r(base).jsValue()), executeData.m_callFrame->r(i).i());
            CHECK_FOR_EXCEPTION();
            executeData.m_callFrame->uncheckedR(i) = Register::withInt(executeData.m_callFrame->r(i).i() + 1);
            if (key) {
                CHECK_FOR_TIMEOUT();
                executeData.m_callFrame->uncheckedR(dst) = key;
                executeData.m_vPC += target;
                NEXT_INSTRUCTION();
            }
        }

        executeData.m_vPC += OPCODE_LENGTH(op_next_pname);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jmp_scopes) {
        /* jmp_scopes count(n) target(offset)

           Removes the a number of items from the current scope chain
           specified by immediate number count, then jumps to offset
           target.
        */
        int count = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;

        ScopeChainNode* tmp = executeData.m_callFrame->scopeChain();
        while (count--)
            tmp = tmp->pop();
        executeData.m_callFrame->setScopeChain(tmp);

        executeData.m_vPC += target;
        NEXT_INSTRUCTION();
    }
#if ENABLE(COMPUTED_GOTO_INTERPRETER)
    // Appease GCC
    goto *(&&skip_new_scope);
#endif
    DEFINE_OPCODE(op_push_new_scope) {
        /* new_scope dst(r) property(id) value(r)
         
           Constructs a new StaticScopeObject with property set to value.  That scope
           object is then pushed onto the ScopeChain.  The scope object is then stored
           in dst for GC.
         */
        executeData.m_callFrame->setScopeChain(createExceptionScope(executeData.m_callFrame, executeData.m_vPC));

        executeData.m_vPC += OPCODE_LENGTH(op_push_new_scope);
        NEXT_INSTRUCTION();
    }
#if ENABLE(COMPUTED_GOTO_INTERPRETER)
    skip_new_scope:
#endif
    DEFINE_OPCODE(op_catch) {
        /* catch ex(r)

           Retrieves the VM's current exception and puts it in register
           ex. This is only valid after an exception has been raised,
           and usually forms the beginning of an exception executeData.m_handler.
        */
        ASSERT(executeData.m_exceptionValue);
        ASSERT(!executeData.m_globalData->exception);
        int ex = executeData.m_vPC[1].u.operand;
        executeData.m_callFrame->uncheckedR(ex) = executeData.m_exceptionValue;
        executeData.m_exceptionValue = JSValue();

        executeData.m_vPC += OPCODE_LENGTH(op_catch);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_throw) {
        /* throw ex(r)

           Throws register ex as an exception. This involves three
           steps: first, it is set as the current exception in the
           VM's internal state, then the stack is unwound until an
           exception executeData.m_handler or a native code boundary is found, and
           then control resumes at the exception executeData.m_handler if any or
           else the script returns control to the nearest native caller.
        */

        int ex = executeData.m_vPC[1].u.operand;
        executeData.m_exceptionValue = executeData.m_callFrame->r(ex).jsValue();

        executeData.m_handler = throwException(executeData.m_callFrame, executeData.m_exceptionValue, executeData.m_vPC - executeData.m_codeBlock->instructions().begin());
        if (!executeData.m_handler)
            END_EXECUTE(throwError(executeData.m_callFrame, executeData.m_exceptionValue))

        executeData.m_codeBlock = executeData.m_callFrame->codeBlock();
        executeData.m_vPC = executeData.m_codeBlock->instructions().begin() + executeData.m_handler->target;
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_throw_reference_error) {
        /* op_throw_reference_error message(k)

           Constructs a new reference Error instance using the
           original constructor, using constant message as the
           message string. The result is thrown.
        */
        UString message = executeData.m_callFrame->r(executeData.m_vPC[1].u.operand).jsValue().toString(executeData.m_callFrame);
        executeData.m_exceptionValue = JSValue(createReferenceError(executeData.m_callFrame, message));
        VM_THROW();
    }
    DEFINE_OPCODE(op_end) {
        /* end result(r)
           
           Return register result as the value of a global or eval
           program. Return control to the calling native code.
        */

        int result = executeData.m_vPC[1].u.operand;
        END_EXECUTE(executeData.m_callFrame->r(result).jsValue())
    }
    DEFINE_OPCODE(op_put_getter) {
        /* put_getter base(r) property(id) function(r)

           Sets register function on register base as the getter named
           by identifier property. Base and function are assumed to be
           objects as this op should only be used for getters defined
           in object literal form.

           Unlike many opcodes, this one does not write any output to
           the register file.
        */
        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int function = executeData.m_vPC[3].u.operand;

        ASSERT(executeData.m_callFrame->r(base).jsValue().isObject());
        JSObject* baseObj = asObject(executeData.m_callFrame->r(base).jsValue());
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        ASSERT(executeData.m_callFrame->r(function).jsValue().isObject());
        baseObj->defineGetter(executeData.m_callFrame, ident, asObject(executeData.m_callFrame->r(function).jsValue()));

        executeData.m_vPC += OPCODE_LENGTH(op_put_getter);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_put_setter) {
        /* put_setter base(r) property(id) function(r)

           Sets register function on register base as the setter named
           by identifier property. Base and function are assumed to be
           objects as this op should only be used for setters defined
           in object literal form.

           Unlike many opcodes, this one does not write any output to
           the register file.
        */
        int base = executeData.m_vPC[1].u.operand;
        int property = executeData.m_vPC[2].u.operand;
        int function = executeData.m_vPC[3].u.operand;

        ASSERT(executeData.m_callFrame->r(base).jsValue().isObject());
        JSObject* baseObj = asObject(executeData.m_callFrame->r(base).jsValue());
        Identifier& ident = executeData.m_codeBlock->identifier(property);
        ASSERT(executeData.m_callFrame->r(function).jsValue().isObject());
        baseObj->defineSetter(executeData.m_callFrame, ident, asObject(executeData.m_callFrame->r(function).jsValue()), 0);

        executeData.m_vPC += OPCODE_LENGTH(op_put_setter);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_method_check) {
        executeData.m_vPC++;
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_jsr) {
        /* jsr retAddrDst(r) target(offset)

           Places the address of the next instruction into the retAddrDst
           register and jumps to offset target from the current instruction.
        */
        int retAddrDst = executeData.m_vPC[1].u.operand;
        int target = executeData.m_vPC[2].u.operand;
        executeData.m_callFrame->r(retAddrDst) = executeData.m_vPC + OPCODE_LENGTH(op_jsr);

        executeData.m_vPC += target;
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_sret) {
        /* sret retAddrSrc(r)

         Jumps to the address stored in the retAddrSrc register. This
         differs from op_jmp because the target address is stored in a
         register, not as an immediate.
        */
        int retAddrSrc = executeData.m_vPC[1].u.operand;
        executeData.m_vPC = executeData.m_callFrame->r(retAddrSrc).vPC();
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_debug) {
        /* debug debugHookID(n) firstLine(n) lastLine(n)

         Notifies the debugger of the current state of execution. This opcode
         is only generated while the debugger is attached.
        */
        int debugHookID = executeData.m_vPC[1].u.operand;
        int firstLine = executeData.m_vPC[2].u.operand;
        int lastLine = executeData.m_vPC[3].u.operand;

        debug(executeData.m_callFrame, static_cast<DebugHookID>(debugHookID), firstLine, lastLine);

        executeData.m_vPC += OPCODE_LENGTH(op_debug);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_profile_will_call) {
        /* op_profile_will_call function(r)

         Notifies the profiler of the beginning of a function call. This opcode
         is only generated if developer tools are enabled.
        */
        int function = executeData.m_vPC[1].u.operand;

        if (*executeData.m_enabledProfilerReference)
            (*executeData.m_enabledProfilerReference)->willExecute(executeData.m_callFrame, executeData.m_callFrame->r(function).jsValue());

        executeData.m_vPC += OPCODE_LENGTH(op_profile_will_call);
        NEXT_INSTRUCTION();
    }
    DEFINE_OPCODE(op_profile_did_call) {
        /* op_profile_did_call function(r)

         Notifies the profiler of the end of a function call. This opcode
         is only generated if developer tools are enabled.
        */
        int function = executeData.m_vPC[1].u.operand;

        if (*executeData.m_enabledProfilerReference)
            (*executeData.m_enabledProfilerReference)->didExecute(executeData.m_callFrame, executeData.m_callFrame->r(function).jsValue());

        executeData.m_vPC += OPCODE_LENGTH(op_profile_did_call);
        NEXT_INSTRUCTION();
    }
    DEFINE_LABEL(vm_throw) {
        executeData.m_globalData->exception = JSValue();
        if (!executeData.m_tickCount) {
            // The executeData.m_exceptionValue is a lie! (GCC produces bad code for reasons I 
            // cannot fathom if we don't assign to the executeData.m_exceptionValue before branching)
            executeData.m_exceptionValue = createInterruptedExecutionException(executeData.m_globalData);
        }
        JSGlobalObject* globalObject = executeData.m_callFrame->lexicalGlobalObject();
        executeData.m_handler = throwException(executeData.m_callFrame, executeData.m_exceptionValue, executeData.m_vPC - executeData.m_codeBlock->instructions().begin());
        if (!executeData.m_handler) {
            // Can't use the callframe at this point as the scopechain, etc have
            // been released.
            END_EXECUTE(throwError(globalObject->globalExec(), executeData.m_exceptionValue));
        }

        executeData.m_codeBlock = executeData.m_callFrame->codeBlock();
        executeData.m_vPC = executeData.m_codeBlock->instructions().begin() + executeData.m_handler->target;
        NEXT_INSTRUCTION();
    }
