/*
 * Copyright (c) 2008, 2009, Google Inc. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "GTXImageDecoder.h"
#include <wtf/PassOwnPtr.h>

#if PLATFORM(WIIU)
#include <wkgfx_GTXLoader.h>
#include <wiiu/shim_memMgr.h>
#include "../../../platform/graphics/wiiu/GraphicsContextPlatformPrivate.h"
#endif

namespace WebCore {

GTXImageDecoder::GTXImageDecoder(ImageSource::AlphaOption alphaOption,
                                 ImageSource::GammaAndColorProfileOption gammaAndColorProfileOption)
    : ImageDecoder(alphaOption, gammaAndColorProfileOption)
{
}

void GTXImageDecoder::setData(SharedBuffer* data, bool allDataReceived)
{
    if (failed())
        return;

    ImageDecoder::setData(data, allDataReceived);
}

bool GTXImageDecoder::isSizeAvailable()
{
    if (!ImageDecoder::isSizeAvailable())
        decode(true);

    return ImageDecoder::isSizeAvailable();
}

ImageFrame* GTXImageDecoder::frameBufferAtIndex(size_t index)
{
    if (index)
        return 0;

    if (m_frameBufferCache.isEmpty()) {
        m_frameBufferCache.resize(1);
        m_frameBufferCache.first().setPremultiplyAlpha(m_premultiplyAlpha);
    }

    ImageFrame* buffer = &m_frameBufferCache.first();
    if (buffer->status() != ImageFrame::FrameComplete)
        decode(false);
    return buffer;
}

bool GTXImageDecoder::setFailed()
{
    return ImageDecoder::setFailed();
}

void GTXImageDecoder::decode(bool onlySize)
{
    if (failed())
        return;

    WKgfx::GTXImageLoader imageLoader;
    if(!imageLoader.LoadGTXFileIntoHeap(static_cast<const void*>(m_data->data()), 0, SHIM_TEXTURE_HEAP))
        return;
    
    setSize(imageLoader.GetWidth(), imageLoader.GetHeight());

    if(onlySize) //we have the size at this point
    {
        imageLoader.FreeImageData();
        imageLoader.FreeHeader();
        return;
    }

    ImageFrame* buffer = &m_frameBufferCache.first();
    buffer->setIsNativeTextureFormat(true);
    buffer->setNativeTextureHeader(imageLoader.GetNativeImgPtr());
    buffer->setOriginalFrameRect(IntRect(0,0,imageLoader.GetWidth(), imageLoader.GetHeight()));

    buffer->setStatus(ImageFrame::FrameComplete);
}

} // namespace WebCore
