/*
 * Copyright (c) 2013 Crank Software, Inc. All rights reserved.
 *
 */

#ifndef TRIE_H_
#define TRIE_H_

#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

#include <cstdio>

namespace WebCore {

template<class T> class Trie {

public:

    Trie(T default_value)
        : default_value(default_value)
    {
    	root.value = default_value;
    }

    virtual ~Trie()
    { 
    }     

    void insert(const String &key, T &t)
    {
        Node *current = &root;
        size_t index = 0;
        bool found = true;

        //printf("in tree: root size: %d\n", current->children.size());
        while (found && index < key.length()) {

            UChar c = key[index];

            found = false;
            for (size_t i = 0; i < current->children.size(); ++i) {
                if (current->children[i].key == c) {
                    //printf("%c", key[index]);
                    current = &current->children[i];
                    ++index;
                    found = true;
                    break;
                }
            } 
        }

        //printf("new nodes:\n");
        while (index < key.length()) {
            Node n;
            n.key = key[index];
            n.value = default_value;
            //printf("%c", key[index]);

            current->children.append(n); 
            current = &current->children.last(); 

            ++index;
        }

        //printf("\n");
        //printf("done\n");

        current->value = t;
    }

    T find(const String &key)
    {

        Node *current = &root;
        size_t index = 0;
        bool found = true;

        //printf("in tree: root size: %d\n", current->children.size());
        while (found && index < key.length()) {

            UChar c = key[index];

            found = false;
            for (size_t i = 0; i < current->children.size(); ++i) {
                if (current->children[i].key == c) {
                    //printf("%c", key[index]);
                    current = &current->children[i];
                    ++index;
                    found = true;
                    break;
                }
            } 
        }

        return current->value; 

    }

private:

    struct Node {
        UChar key;
        T value;
        Vector<Node> children; 
    };

    Node root;
    T default_value;

};

}

#endif
